/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: allocate_crm_optimal_crd4.sas
 *
 * PURPOSE: Allocate CRMs through PROC NETFLOW and post-process the results. 
 *
 *
 *   USAGE: %allocate_crm_optimal_crd4(EXPCRMDS=, EXPDS=, NODEDS=, ARCDS=, EXPNOCRM=, 
 *                                EXP1CRM1=, CONDS=, CONOUTDS=, OUTDS=);
 *          EXPCRMDS: linked exposure-crm data set
 *          EXPDS   : exposure data set
 *          NODEDS  : node data set
 *          ARCDS   : arc data set
 *          EXPNOCRM: exposures without crm
 *          EXP1CRM1: unique exposure-crm links
 *          CONDS   : PROC INTPOINT constraint data set
 *          CONOUTDS: PROC INTPOINT output data set
 *          OUTDS   : data set for the final output
 *
 *   NOTES:  
 *          
 *
 *****************************************************************************/

%macro allocate_crm_optimal_crd4(EXPCRMDS=, EXPDS=, NODEDS=, ARCDS=, EXPNOCRM=, EXP1CRM1=, 
                            CONDS=, CONOUTDS=, OUTDS=);

/*- 1: allocate CRMs: a generalized network problem -*/
    /*- Using INTPOINT before the generelized netflow is productionized -*/
    /*- convert the node and arc data to constraint data -*/
    %local NARCS NCONS NCOEFS UPDATED;
    %let UPDATED = 0;
      
 %do inetwork=0 %to &N_NETWORK-1;
    data condata_node&inetwork (keep=_TYPE_ _ROW_ _COL_ _COEF_);
    length _TYPE_ $8. _ROW_ $32. _COL_ $80.;    
    set &NODEDS.&inetwork;
    if _NODE_ NE 'NOT_COVERED';

    _ROW_ = _NODE_;
    _COL_ = '';
    _COEF_ = _SUPDEM_ * &NETFLOW_SCALE_FACTOR; /* adjust for scaling factor */
    if _TYPE_ = 'EXP' then _TYPE_ = 'EQ';
    else _TYPE_ = 'LE';
    run;

    data condata_arc&inetwork;
    length _TYPE_ $3. _ROW_ $32. _COL_ $80.;        
    set &ARCDS.&inetwork ;
    keep _TYPE_ _ROW_ _COL_ _COEF_;
    
    _COL_ = _ARCID_;
    _TYPE_ = '';
    _ROW_ = CRM_ID;
    _COEF_ = 1;
    if _ROW_ NE 'NOT_COVERED' then 
       output;
    
    _ROW_ = EXP_ID;
    _COEF_ = -_MULT_;
    output;
    
    _TYPE_ = 'MIN';
    _ROW_ = 'OBJECTIVE';
    if _COST_ NE 0 then do;
       _COEF_ = _COST_;
       output;
    end;
   
    run;
    
    data &CONDS.&inetwork;
    set condata_node&inetwork condata_arc&inetwork;
    run;
    
    %let NARCS  = %eval(%attrn(&ARCDS.&inetwork, NOBS) * 2);
    %let NCOEFS = %attrn(&CONDS.&inetwork, NOBS);
    %let NCONS  = %eval(%sysfunc(floor(&NCOEFS*0.5)) + 1);
    
/*- run optimization only if there is a meanful network -*/
    %if &NARCS > 0 %then %do;
        /*- get mps input data -*/
        %lp_to_mps(lp=&CONDS.&inetwork, mps=mpsconds&inetwork, format=SPARSE);
        /*- using proc optlp -*/
        proc optlp maxiter=&NETFLOW_MAX_ITERATION 
                   data = mpsconds&inetwork 
                   solver = &NETFLOW_LP_SOLVER
                   primalout= conoutds&inetwork ;
        run;
        
        /*- check for failure -*/
        data _null_;
        istatus = index("&_OROPTLP_", "STATUS=OK");
        if istatus = 0 then
           call symput("SYSTEM_FATAL_ERROR", "Y");
        run;
        
        /*- check system fatal error -*/
        %if &SYSTEM_FATAL_ERROR=Y %then %do;
            %print_msg(WARNING, 2, 230);
            %goto EXIT;
        %end;
        
        proc sort data=&ARCDS.&inetwork;
        by _ARCID_;
        run;
        
        proc sort data=conoutds&inetwork;
        by _VAR_;
        run;

        /*- get some information back into arcout -*/ 
        data arcout&inetwork;                                                                                                                      
        merge &ARCDS.&inetwork (in=arc)                                                                                                
              conoutds&inetwork (in=out keep=_VAR_ _OBJCOEF_ _VALUE_ rename=(_VAR_=_ARCID_)); 
        by _ARCID_; 
        if arc;
        _FCOST_ = _VALUE_*_OBJCOEF_;    
        /* scale back from netflow result */
        if out then do;
           _VALUE_ = _VALUE_ / &NETFLOW_SCALE_FACTOR; 
           _FCOST_ = _FCOST_ / &NETFLOW_SCALE_FACTOR; 
        end;
        run;  
        
        %if &UPDATED=0 %then %do;
            data &CONOUTDS;
            set conoutds&inetwork;
            run;
            
            data arcout;
            set arcout&inetwork;
            run;
            
            data node0(keep = _NODE_ _TYPE_  _SUPDEM_);;
            set &NODEDS.&inetwork;
            run;
            %let UPDATED = 1;
        %end;
        %else %do;
      proc append base=&CONOUTDS data=conoutds&inetwork;
            run;
            
            proc append base=arcout data=arcout&inetwork;
            run;
            
            proc append base=node0 data=&NODEDS.&inetwork;
            run;
        %end;
    %end;
 %end;
  
 %if &N_NETWORK > 0 %then %do;
    /*- update with crm supply -*/
    proc sort data = arcout; 
    by CRM_ID;
    run;

    proc sort data = node0;
    by _NODE_;
    run;

    data arcout(drop = _TYPE_ _SUPDEM_);
    merge arcout(in=a) node0(where = (_TYPE_ = 'CRM') rename = (_NODE_ = CRM_ID));
    by CRM_ID ;
    if a;
    if _SUPPLY_ = . then _SUPPLY_ = _SUPDEM_;
    run;

    /*- update with exposure demand -*/
    proc sort data = arcout; by EXP_ID;run;

    data arcout(drop = _TYPE_ _SUPDEM_);
    merge arcout(in=a) node0(where = (_TYPE_ = 'EXP') rename = (_NODE_ = EXP_ID));
    by EXP_ID;
    if a;
    if _DEMAND_ = . then do;
        _DEMAND_ = - _SUPDEM_;
    end;
    run;

/*- 3: adjust flow and flow cost -*/
/*- get exposure haircut -*/
    proc sql;
    create table arcout_ as
    select arcout.*, exp_crm.HAIRCUT_FX, exp_crm.HAIRCUT_EXP, exp_crm.CRM_TYPE
    from arcout arcout
    left join &EXPCRMDS as exp_crm
    on arcout._ARCID_ = exp_crm._ARCID_
    order by EXP_ID;
    quit;

/*- adjust flow cost for multiplier for covered exposures and -*/
/*- exposure haircut for uncovered portion of exposures with crms in other observations -*/

/*- get all exposures adjusted for haircut -*/
    data exp_haircut_adjust(keep = EXP_ID HAIRCUT_EXP rename=(HAIRCUT_EXP=H));
    set arcout_(where = (index(_ARCID_, '***H_E') >0 and _VALUE_ >0));
    run;

    data arcout_(drop = HAIRCUT_EXP _MULT_ H);
    merge arcout_(in=arcout) exp_haircut_adjust(in=h_adjust);
    by EXP_ID;

    _SUPPLY_ = _SUPPLY_ ;
    _DEMAND_ = _DEMAND_ ;
    _VALUE_   = _VALUE_ ;
    _FCOST_  = _FCOST_  ;
    _VALUE_0  = _VALUE_; /*- unadjusted flow -*/

    if CRM_ID NE 'NOT_COVERED' then 
        _VALUE_  = _VALUE_ * _MULT_;  

    if h_adjust then do;
        if CRM_TYPE in ('FINANCIAL_COLLATERAL', 'NETTING') then 
           _VALUE_ = _VALUE_ * sum(1, H / 100);

        if CRM_ID = 'NOT_COVERED' then do;
           _VALUE_ = _VALUE_ * sum(1, H / 100);
           _FCOST_ = _FCOST_ * sum(1, H / 100);
        end;
    end;
  
    run;

/*- 4: get detailed information on exposures and crms -*/
    proc sql;
    /*- for exposures with real crm links -*/
    create table arcout_1 as
    select arcout_._COST_, arcout_._SUPPLY_, arcout_._DEMAND_, arcout_._VALUE_, arcout_._VALUE_0, arcout_._FCOST_, exp_crm.* 
    from arcout_ as arcout_, &EXPCRMDS(drop=_COST_) as exp_crm 
    where arcout_._ARCID_  = exp_crm._ARCID_;
        
    create table arcout_2 as
    select arcout_.CRM_ID, arcout_._COST_, arcout_._SUPPLY_, arcout_._DEMAND_, arcout_._VALUE_, arcout_._VALUE_0, arcout_._FCOST_, exp.* 
    from arcout_ as arcout_, &EXPDS as exp
    where arcout_.EXP_ID = exp.EXP_ID and arcout_._ARCID_ not in (select _ARCID_ from arcout_1);
    quit;
 %end;
  
 %if N_EXP_CRM > 0 %then %do;
/*- 5: process unique exposure and crm pairs -*/
    data arcout_3 (drop=tempvalue)
         exp_haircut_adjust2 (keep=EXP_ID HAIRCUT_EXP rename=(HAIRCUT_EXP=H));
    set &EXP1CRM1;
    length _ARCID_ $80.;
    /*- secured portion -*/
    _SUPPLY_ = CRM_VALUE;

    if APPROACH = 'STD' then do;
       %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
           if OFF_BALANCE_SHEET_FLG=1 then 
              _DEMAND_ = max(0, sum(EAD, -SPECIFIC_PROVISION*CCF/100));
           else
              _DEMAND_ = max(0, sum(EAD, -SPECIFIC_PROVISION));
       %end;
       %else %do;
           _DEMAND_ = max(0, sum(EAD, -SPECIFIC_PROVISION));
       %end;
    end;
    else _DEMAND_ = EAD;
    
    /*- no need for mitigation -*/
    if ( _COST_ > RW_EXP or _MULT_ = 0 or _SUPPLY_ = 0 ) then do;
       _VALUE_0 = 0;
       tempvalue= 0;
       _VALUE_  = 0;
       _FCOST_  = 0;
    end;
    /*- applying crm -*/
    else do;
       _VALUE_0 = min(_SUPPLY_, _DEMAND_/_MULT_);
       tempvalue= _VALUE_0 * _MULT_;
       _VALUE_  = tempvalue;
       if index(_ARCID_, '***H_E') >0 then do;
          _VALUE_ = _VALUE_ * sum(1, HAIRCUT_EXP / 100);
          output exp_haircut_adjust2;
       end;
       _FCOST_ = _VALUE_0 * _COST_;
    end;
    output arcout_3 ;
    /*- unsecured portion -*/   
    CRM_ID   = 'NOT_COVERED';
    CRM_TYPE = 'NOT_COVERED';
    _SUPPLY_ = 0;
    _VALUE_  = _DEMAND_ - tempvalue;
    _FCOST_  = _VALUE_ * RW_EXP ;
    _VALUE_0 = _VALUE_; 
    if index(_ARCID_, '***H_E') >0 then do;
       _VALUE_ = _VALUE_ * sum(1, HAIRCUT_EXP / 100);
       _FCOST_ = _FCOST_ * sum(1, HAIRCUT_EXP / 100);
    end;
    _COST_   = RW_EXP;
    _MULT_   = 1;
    _ARCID_  = compress(EXP_ID || 'NOT_COVERED');
    output arcout_3 ;
    
    run;
    
    proc append base=exp_haircut_adjust data=exp_haircut_adjust2;
    run;
    
    proc sort data=exp_haircut_adjust nodupkey;
    by EXP_ID;
    run;
 %end;
    

/*- 6: process exposures without crm -*/
    data arcout_4;
    set &EXPNOCRM;
    length _ARCID_ $80. CRM_ID $32.;
    _SUPPLY_ = 0;
    if APPROACH = 'STD' then do;
       %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
           if OFF_BALANCE_SHEET_FLG=1 then 
              _DEMAND_ = max(0, sum(EAD, -SPECIFIC_PROVISION*CCF/100));
           else
              _DEMAND_ = max(0, sum(EAD, -SPECIFIC_PROVISION));
       %end;
       %else %do;
           _DEMAND_ = max(0, sum(EAD, -SPECIFIC_PROVISION));
       %end;
    end;
    else _DEMAND_ = EAD;
    
    if LGD_CRM_ADJUSTED_FLG ne 1 then do ; 
    CRM_ID   = 'NOT_COVERED';
    CRM_TYPE = 'NOT_COVERED';
    end; 
    _VALUE_  = _DEMAND_ ;
    _FCOST_  = _VALUE_ * RW_EXP ;
    _VALUE_0 = _VALUE_;
    _MULT_   = 1;
    _COST_   = RW_EXP;
    _ARCID_  = compress(EXP_ID || 'NOT_COVERED');
    run;    
    
/*- 7: now we have all exposure and crm information -*/
    %if &N_EXP_CRM=0 %then %do;
        data arcout_temp;
        set &EXPCRMDS;
        stop;
        run;
    %end;
    data &OUTDS;
    set %if &N_NETWORK > 0 %then %do; arcout_1 arcout_2 %end;
        %if &N_EXP_CRM > 0 %then %do; arcout_3 %end;
        %else %do; arcout_temp %end;
        arcout_4;
    run;

    proc sort data = &OUTDS;
    by EXP_ID;
    run;

%EXIT:
%mend allocate_crm_optimal_crd4;
