/*****************************************************************************
 * Copyright (c) 2013 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: calc_sec_rwa_adj_amt_crd4.sas
 *
 * PURPOSE: Calculate the RWA and RWA adjustment amounts for securitization exposures:  
 *          1. maturity mismatch for synthetic securitization exposures: CRR Articles 249-250.
 *          2. before_ and after_ cap RWA for all originators' securitization exposures: CRR Article 252.
 *          3. before_ and after_ cap additional RWA for exposures with EAP: CRR Article 256.
 *          
 *   USAGE: calc_sec_rwa_adj_amt_crd4.sas
 *          
 *
 *   NOTES: 
 *
 *
 *****************************************************************************/
%macro calc_sec_rwa_adj_amt_crd4(calc_option=) ;
   
    if SECURITIZATION_EXPOSURE_FLG =1 then do ; 
            
       %if &calc_option= CRM_ALLOCATION %then %do ;
                             
            /*- calculate maturity mismatch adjustment to RWA:  CRR Articles 249-250  -*/
            if PRIMARY_BANK_ROLE = 'ORIGINATOR' and SEC_STRUCTURE_TYPE = 'SYNTHETIC' and not missing(MAT_MISMATCH_WEIGHT_SEC) then do;
               if not missing(POOL_FACE_VALUE_AMT) then 
                  PRO_RATA = (EAD_PORTION/EAD_PORTION_SUM) * (EXP_VALUE/POOL_FACE_VALUE_AMT) ;
               else if not missing(EAD_SEC_UNDERLYING_SUM) then 
                  PRO_RATA = (EAD_PORTION/EAD_PORTION_SUM) * (EXP_VALUE/EAD_SEC_UNDERLYING_SUM) ;

                 RWA_SEC_MAT_MIS_ADJ_AMT = sum(RWA * MAT_MISMATCH_WEIGHT_SEC, RWA_SEC_UNDERLYING_SUM * MAT_MISMATCH_WEIGHT_SEC_UNDERLY * PRO_RATA, -RWA);
                 RWA = RWA + RWA_SEC_MAT_MIS_ADJ_AMT ;
            end;
            
            /*- begin Cap calculation for securitizations -*/
            RWA_BEFORE_CAP = RWA;
            if PRIMARY_BANK_ROLE in ('ORIGINATOR', 'SPONSOR') and STD_PRODUCT_SUBCLASS not in ('EAP_INVESTORS_INT', 'DEDUCTION', 'EAP_ORIGINATOR_INT') and
               IRB_PRODUCT_SUBCLASS not in ('EAP_INVESTORS_INT', 'DEDUCTION', 'EAP_ORIGINATOR_INT') then do;
                /*- calculate cap adjustment to RWA based on underlying RWAs:  CRR Articles 252  -*/
                if missing(RWA_SEC_UNDERLYING_SUM_FOR_CAP) then RWA_SEC_UNDERLYING_SUM_FOR_CAP = POOL_FACE_VALUE_AMT * AVG_RISK_WEIGHT/100; 
                
                if not missing(RWA_SEC_UNDERLYING_SUM_FOR_CAP) and  RWA_SEC_UNDERLYING_SUM_FOR_CAP < RWA_SEC_SUM then do;
                    percentage_reduction = RWA_SEC_UNDERLYING_SUM_FOR_CAP/RWA_SEC_SUM;
                    RWA = percentage_reduction * RWA_BEFORE_CAP;
                end;
            end;
            
            /*- additional RWA in respect of the exposure value of the originator's interest should be                 -*/
            /*- pro rata exposure to the securitised exposures as if they had not been securitised: CRR Article 256(2) -*/
            else if PRIMARY_BANK_ROLE ='ORIGINATOR' and (STD_PRODUCT_SUBCLASS ='EAP_ORIGINATOR_INT' or IRB_PRODUCT_SUBCLASS ='EAP_ORIGINATOR_INT') then do;
                   if not missing(POOL_FACE_VALUE_AMT) then do ;
                      if missing(RWA_SEC_UNDERLYING_SUM) then RWA_SEC_UNDERLYING_SUM = POOL_FACE_VALUE_AMT * AVG_RISK_WEIGHT / 100;    
                      RWA = RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / POOL_FACE_VALUE_AMT *(1- INVESTORS_INTEREST_PCT / 100) ;
                   end ;   
                   
                   else if not missing(EAD_SEC_UNDERLYING_SUM) and not missing(RWA_SEC_UNDERLYING_SUM) then 
                         RWA = RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / EAD_SEC_UNDERLYING_SUM *(1- INVESTORS_INTEREST_PCT / 100) ;
                   
                   RWA_BEFORE_CAP = RWA;
            end ;
            
            /*- calculate cap adjustment to additional RWA in respect of the exposure value of investors' interest:  CRR Articles 256(4)  -*/ 
            else if PRIMARY_BANK_ROLE ='ORIGINATOR' and (STD_PRODUCT_SUBCLASS ='EAP_INVESTORS_INT' or IRB_PRODUCT_SUBCLASS ='EAP_INVESTORS_INT') then do;                                         
                                     
                   if not missing(POOL_FACE_VALUE_AMT) then do ;
                      if missing(RWA_SEC_UNDERLYING_SUM) then RWA_SEC_UNDERLYING_SUM = POOL_FACE_VALUE_AMT * AVG_RISK_WEIGHT / 100;                       
                      RWA = min(RWA_BEFORE_CAP, 
                             max((RWA_ALL_TRANCHES - RWA_TRANCHE_SUM) * TOTAL_REVOLVING / POOL_FACE_VALUE_AMT, 
                                  RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / POOL_FACE_VALUE_AMT * INVESTORS_INTEREST_PCT / 100)) ;
                   end ;
                   
                   else if not missing(EAD_SEC_UNDERLYING_SUM) and not missing(RWA_SEC_UNDERLYING_SUM)  then 
                      RWA = min(RWA_BEFORE_CAP, 
                             max((RWA_ALL_TRANCHES - RWA_TRANCHE_SUM) * TOTAL_REVOLVING / EAD_SEC_UNDERLYING_SUM, 
                                  RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / EAD_SEC_UNDERLYING_SUM * INVESTORS_INTEREST_PCT / 100)) ;
            end;
       %end ;

       %else %if &calc_option= CAPITAL %then %do ;
            
            /*- calculate maturity mismatch adjustment to RWA:  CRR Articles 249-250  -*/
            if PRIMARY_BANK_ROLE = 'ORIGINATOR' and SEC_STRUCTURE_TYPE = 'SYNTHETIC' and not missing(MAT_MISMATCH_WEIGHT_SEC) then do;             
                if not missing(POOL_FACE_VALUE_AMT) then PRO_RATA = EXP_VALUE / POOL_FACE_VALUE_AMT ;
                else if not missing(EAD_SEC_UNDERLYING_SUM) then PRO_RATA = EXP_VALUE / EAD_SEC_UNDERLYING_SUM ; 
            
                RWA_SEC_MAT_MIS_ADJ_AMT = sum(RWA * MAT_MISMATCH_WEIGHT_SEC, RWA_SEC_UNDERLYING_SUM * MAT_MISMATCH_WEIGHT_SEC_UNDERLY * PRO_RATA, -RWA);
                RWA = RWA + RWA_SEC_MAT_MIS_ADJ_AMT ;
            end;
            
            /*- begin Cap calculation for securitizations -*/
            RWA_BEFORE_CAP = RWA;
            if PRIMARY_BANK_ROLE in ('ORIGINATOR', 'SPONSOR') and STD_PRODUCT_SUBCLASS not in ('EAP_INVESTORS_INT', 'DEDUCTION', 'EAP_ORIGINATOR_INT') and
               IRB_PRODUCT_SUBCLASS not in ('EAP_INVESTORS_INT', 'DEDUCTION', 'EAP_ORIGINATOR_INT') then do;
                /*- calculate cap adjustment to RWA based on underlying RWAs:  CRR Articles 252  -*/
                if missing(RWA_SEC_UNDERLYING_SUM_FOR_CAP) then RWA_SEC_UNDERLYING_SUM_FOR_CAP = POOL_FACE_VALUE_AMT * AVG_RISK_WEIGHT/100; 
                
                if not missing(RWA_SEC_UNDERLYING_SUM_FOR_CAP) and  RWA_SEC_UNDERLYING_SUM_FOR_CAP < RWA_SEC_SUM then do;
                    percentage_reduction = RWA_SEC_UNDERLYING_SUM_FOR_CAP/RWA_SEC_SUM;
                    RWA = percentage_reduction * RWA_BEFORE_CAP;
                end;
            end;
            
            /*- additional RWA in respect of the exposure value of the originator's interest should be                 -*/
            /*- pro rata exposure to the securitised exposures as if they had not been securitised: CRR Article 256(2) -*/
            else if PRIMARY_BANK_ROLE ='ORIGINATOR' and (STD_PRODUCT_SUBCLASS ='EAP_ORIGINATOR_INT' or IRB_PRODUCT_SUBCLASS ='EAP_ORIGINATOR_INT') then do;
                   if not missing(POOL_FACE_VALUE_AMT) then do ;
                      if missing(RWA_SEC_UNDERLYING_SUM) then RWA_SEC_UNDERLYING_SUM = POOL_FACE_VALUE_AMT * AVG_RISK_WEIGHT / 100; 
                      RWA = RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / POOL_FACE_VALUE_AMT * (1- INVESTORS_INTEREST_PCT /100) ;                     
                   end ;  
                   
                   else if not missing(EAD_SEC_UNDERLYING_SUM) and not missing(RWA_SEC_UNDERLYING_SUM) then
                       RWA = RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / EAD_SEC_UNDERLYING_SUM * (1- INVESTORS_INTEREST_PCT /100) ;
                   
                   RWA_BEFORE_CAP = RWA;    
            end ;
            
             /*- calculate cap adjustment to additional RWA in respect of the exposure value of investors' interest:  CRR Articles 256(4)  -*/
            else if PRIMARY_BANK_ROLE ='ORIGINATOR' and (STD_PRODUCT_SUBCLASS ='EAP_INVESTORS_INT' or IRB_PRODUCT_SUBCLASS ='EAP_INVESTORS_INT') then do;                 
                   if not missing(POOL_FACE_VALUE_AMT) then do ;
                      if missing(RWA_SEC_UNDERLYING_SUM) then RWA_SEC_UNDERLYING_SUM = POOL_FACE_VALUE_AMT * AVG_RISK_WEIGHT/100;
                   
                      RWA = min(RWA_BEFORE_CAP, 
                             max((RWA_ALL_TRANCHES - RWA_TRANCHE_SUM) * TOTAL_REVOLVING / POOL_FACE_VALUE_AMT, 
                                  RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / POOL_FACE_VALUE_AMT * INVESTORS_INTEREST_PCT /100)) ;
                   end ;
                   
                   else if not missing(EAD_SEC_UNDERLYING_SUM) and not missing(RWA_SEC_UNDERLYING_SUM) then
                      
                      RWA = min(RWA_BEFORE_CAP, 
                             max((RWA_ALL_TRANCHES - RWA_TRANCHE_SUM) * TOTAL_REVOLVING / EAD_SEC_UNDERLYING_SUM, 
                                  RWA_SEC_UNDERLYING_SUM * TOTAL_REVOLVING / EAD_SEC_UNDERLYING_SUM * INVESTORS_INTEREST_PCT /100)) ;
            
            end;
             
            if RWA > 0 then do ;             
                RWA_UNSECURED = RWA_UNSECURED /sum(RWA_UNSECURED,RWA_SECURED) * RWA ;
                RWA_SECURED = RWA - RWA_UNSECURED ;
            end;
                   
       %end ; 

/*- calculate memorandum item for securitization exposures:               -*/
/*- memorandum item is the risk weighted exposure amount                  -*/
/*- stemming from exposures redistributed to the risk mitigant provider.  -*/
       RWA_SEC_MEMO_AMT = CRM_USED_AMT * RW_EXP_BEFORE_INFRINGEMENT / 100 ;

    end ;   

 %mend calc_sec_rwa_adj_amt_crd4 ;         
          
