/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: d00_pm_mtm_crd4.sas
 *
 * PURPOSE: Define instrument market-to-market pricing methods.
 *
 *   USAGE: 
 *
 *   NOTES: 1. In mark-to-market pricing, values for the following items need
 *             to be computed:
 *             MtM value, EAD, CCF, ADD-ON, haircut and effective maturity
 *          
 *               
 *****************************************************************************/

%macro d00_pm_mtm_crd4;

proc compile outlib  = RD_TEMP.&RD_TEMPLATE
             inlib   = RD_TEMP.&RD_TEMPLATE
             package = mtm
             env     = RD_TEMP.&RD_TEMPLATE
             label   = 'Mark to Market Pricing Functions/Methods';

/*- mark-to-market pricing method for priced accounts -*/
method PM_MTM_ACCOUNT  kind = price ;

/*- initialization -*/
    %initialize_mtm_outvars;

/*- haircut -*/
    %get_haircut;

    _VALUE_ = sum(BALANCE_AMT, INTEREST_AMT);
    
    if COLLATERAL then do;  
        if PROTECTION_VALUE_AMT NE . then do;
            if _VALUE_ NE . then _VALUE_ = min(PROTECTION_VALUE_AMT/HOLDING, _VALUE_);
            else _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
        end;
        OUTV_CRM = _VALUE_;
    end;
    else do;
        OUTV_RESIDUAL_VALUE = LEASE_RESIDUAL_VALUE_AMT;
        /*- use user supplied EAD if supplied -*/
        if not missing(EAD_AMT) then
           OUTV_EAD = EAD_AMT/holding;
        else
           OUTV_EAD = _VALUE_;
        %get_m;
        %check_m;
    end;
    
endmethod;

/*- mark-to-market pricing method for priced instruments -*/
method PM_MTM_INSTRUMENT kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;

/*- haircut selection differs for repurchase style instruments -*/
if (STD_PRODUCT_CLASS = 'REPO_STYLE' or IRB_PRODUCT_CLASS = 'REPO_STYLE') then do ;
    %get_haircut(REPO_EXP=Y)
end ;
else do ;
    %get_haircut 
end ;    
    
/*- add-on -*/
    %get_addon_crd4;

    _VALUE_ = VALUE_AMT;

/*- for master netting treatment of repo-style transactions and market-related off-balance-sheet items -*/
    OUTV_FACE_VALUE = FACE_VALUE_AMT;

    if COLLATERAL then do;
        if PROTECTION_VALUE_AMT NE . then do;
            if _VALUE_ NE . then _VALUE_ = min(PROTECTION_VALUE_AMT/HOLDING, _VALUE_);
            else _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
        end;
        OUTV_CRM = _VALUE_;
    end;
    else do;
        %get_m;
        %check_m;
        
        /*- use user supplied EAD if supplied -*/
        if not missing(EAD_AMT) then
           OUTV_EAD = EAD_AMT;
        else do;        
           if PE_APPROACH = 'CE' then
             OUTV_EAD = max(0, _VALUE_) + FACE_VALUE_AMT * OUTV_ADD_ON / 100;
           else if PE_APPROACH = 'OE' then
             OUTV_EAD = FACE_VALUE_AMT * OUTV_ADD_ON / 100; 
           else 
             OUTV_EAD = _VALUE_ ;
        end;
    end;    

endmethod;

/*- mark-to-market pricing method for facilities -*/
method PM_MTM_FACILITY  kind = price ;

/*- initialization -*/
    %initialize_mtm_outvars;

/*- haircut -*/
    %get_haircut;

/*- CCF -*/
    if CCF_PCT NE . and APPROACH in ('IRBA', 'IRB') then do;
        OUTV_CCF = CCF_PCT;
    end ;
    else do ;
        %get_ccf;
    end;
 
    _VALUE_ =  LIMIT_AMT;
    OUTV_FACE_VALUE = LIMIT_AMT;
    %get_m;
    %check_m;
    
    /*- use user supplied EAD if supplied -*/
    if not missing(EAD_AMT) then
       OUTV_EAD = EAD_AMT/HOLDING;
    else
       OUTV_EAD = _VALUE_ * OUTV_CCF / 100; 

endmethod;

/*- mark-to-market pricing method for equities -*/
method PM_MTM_EQUITY kind=price ;

/*- initialization -*/
    %initialize_mtm_outvars;

/*- haircut -*/
    %get_haircut;

/*- For equity exposures under the standardized approach -*/
/*- a CCF is needed.  -*/
   %get_ccf ;

    _VALUE_ = EQUITY_REF.ISSUE_CODE  ;
    
/*- deduction of equity holdings in financial sector entities: CRR Article 36 -*/ 
   if COUNTERPARTY_CLASS = 'INSTITUTION' and EQ_EXP_FIN_SECT_NO_DEDUCT_FLG ne 1 then do ;      
      OUTV_DEDUCTION_AMT = _VALUE_ ;
      _VALUE_ = 0 ;
   end ;   
        
    if COLLATERAL then do;
        if PROTECTION_VALUE_AMT NE . then do;
            if _VALUE_ NE . then _VALUE_ = min(PROTECTION_VALUE_AMT/HOLDING, _VALUE_);
            else _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
        end;
        OUTV_CRM = _VALUE_;
    end;
    else do;
        OUTV_MAT_EFF = &IRB_M_EQUITY;
        /*- use user supplied EAD if supplied -*/
        if not missing(EAD_AMT) then
           OUTV_EAD = EAD_AMT/HOLDING;
        else
           OUTV_EAD = _VALUE_;
    end;

endmethod;

/*- mark-to-market pricing method for CIU -*/
method PM_MTM_CIU kind=price ;

/*- initialization -*/
    %initialize_mtm_outvars;

    _VALUE_ = FUND_REF.ISSUE_CODE  ;

    if COLLATERAL then do;
        if PROTECTION_VALUE_AMT NE . then do;
            if _VALUE_ NE . then _VALUE_ = min(PROTECTION_VALUE_AMT/HOLDING, _VALUE_);
            else _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
        end;
        OUTV_CRM = _VALUE_;
    end;
    else do;
        /*- use user supplied EAD if supplied -*/
        if not missing(EAD_AMT) then
           OUTV_EAD = EAD_AMT/HOLDING;
        else
           OUTV_EAD = _VALUE_;
    end;

endmethod;

/*- mark-to-market pricing method for fixed coupon bond -*/
method PM_MTM_BOND kind=price;

/*- initialization -*/
    %initialize_mtm_outvars;

/*- haircut selection differs for repurchase style instruments -*/
if (STD_PRODUCT_CLASS = 'REPO_STYLE' or IRB_PRODUCT_CLASS = 'REPO_STYLE') then do ;
    %get_haircut(REPO_EXP=Y)
end ;
else do ;
    %get_haircut 
end ;    

    _VALUE_ = BOND_CF_PRC( _DATE_,            
                           FACE_VALUE_AMT,     
                           PAYMENT_PER_ANNUM_NO,
                           PAYMENT_PER_ANNUM_RT,
                           MATURITY_DATE,       
                           IR_CURVE_REF_GOVT.CURRENCY,
                           IR_CURVE_REF_GOVT.CURRENCY.MAT
                          );
    OUTV_FACE_VALUE = FACE_VALUE_AMT;
    if COLLATERAL then do;
        if PROTECTION_VALUE_AMT NE . then do;
            if _VALUE_ NE . then _VALUE_ = min(PROTECTION_VALUE_AMT/HOLDING, _VALUE_);
            else _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
        end;
        OUTV_CRM = _VALUE_;
    end;
    else do;
        if not missing(EFFECTIVE_MATURITY_YEAR) then
           OUTV_MAT_EFF = EFFECTIVE_MATURITY_YEAR;
        else do;
           OUTV_MAT_EFF = BOND_EFF_MAT( _DATE_,
                                     FACE_VALUE_AMT,
                                     PAYMENT_PER_ANNUM_NO,
                                     PAYMENT_PER_ANNUM_RT,
                                     MATURITY_DATE        
                                   );
           
           OUTV_MAT_EFF = min(&IRBA_M_CAP, max(&IRBA_M_FLOOR, OUTV_MAT_EFF)); 
        end;
        %check_m;
        
        /*- use user supplied EAD if supplied -*/
        if not missing(EAD_AMT) then
           OUTV_EAD = EAD_AMT/HOLDING;
        else
           OUTV_EAD = _VALUE_;
    end;

endmethod;

/*- mark-to-market pricing method for receivables -*/
method PM_MTM_RECEIVABLE kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;

/*- haircut -*/
    %get_haircut;

/*- CCF -*/
    %get_ccf;

    _VALUE_  = VALUE_AMT;
    OUTV_FACE_VALUE = FACE_VALUE_AMT;
    if COLLATERAL then do;
        if PROTECTION_VALUE_AMT NE . then do;
            if _VALUE_ NE . then _VALUE_ = min(PROTECTION_VALUE_AMT/HOLDING, _VALUE_);
            else _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
        end;
        OUTV_CRM = _VALUE_;
    end;
    else do;
        if RECEIVABLE_POOL_FLG NE 1 then do;
            %get_m;
            %check_m;
        end;
        else do;
            if WA_MATURITY_MONTH NE . then
                OUTV_MAT_EFF = min(&IRBA_M_CAP, max(&IRBA_M_FLOOR, WA_MATURITY_MONTH/12));
            else
                OUTV_MAT_EFF = &IRBA_M_DEFAULT;
        end;
        /*- use user supplied EAD if supplied -*/
        if not missing(EAD_AMT) then
           OUTV_EAD = EAD_AMT/HOLDING;
        else do;
           if OUTV_CCF NE . then
              OUTV_EAD = _VALUE_ + sum(FACE_VALUE_AMT, -_VALUE_)* OUTV_CCF / 100;             
           else
              OUTV_EAD = _VALUE_;
        end;
    end;
      
endmethod;

/*- mark-to-market for securitization pool exposures -*/
method PM_MTM_SEC_POOL kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;
    
/*- maturity -*/
    OUTV_MAT_EFF = .;
    _VALUE_ = 0;
    OUTV_FACE_VALUE = POOL_FACE_VALUE_AMT;
    
/*- pool exposure subject to early amortization -*/
    if STD_PRODUCT_SUBCLASS in ('EAP_ORIGINATOR_INT','EAP_INVESTORS_INT') or 
       IRB_PRODUCT_SUBCLASS in ('EAP_ORIGINATOR_INT','EAP_INVESTORS_INT') then do;
 
      /*- originator interest: CRR article 256(2) -*/
       if STD_PRODUCT_SUBCLASS = 'EAP_ORIGINATOR_INT' or IRB_PRODUCT_SUBCLASS = 'EAP_ORIGINATOR_INT' then do ;
           _VALUE_ = TOTAL_REVOLVING * (1 - INVESTORS_INTEREST_PCT / 100);
           OUTV_EAD = _VALUE_ ;
       end ;   
          
      /*- investors' interest: CRR article 256(4) -*/ 
       else if STD_PRODUCT_SUBCLASS = 'EAP_INVESTORS_INT' or IRB_PRODUCT_SUBCLASS = 'EAP_INVESTORS_INT' then do ;   
          /*- CCF -*/
          %get_ccf_sec_eap;
           _VALUE_ = TOTAL_REVOLVING * INVESTORS_INTEREST_PCT / 100;
           OUTV_EAD = _VALUE_ * OUTV_CCF / 100;
       end;       
    end;
    
/*- pool exposure with gain on sale and credit enhancing i/os -*/
    if STD_PRODUCT_SUBCLASS = 'DEDUCTION' or IRB_PRODUCT_SUBCLASS = 'DEDUCTION' then
       OUTV_EAD = sum(GAIN_ON_SALE_AMT, CRDT_ENHNC_INT_ONLY_STRIP_AMT);
    
endmethod;

/*- valuate securitization exposures -*/
method PM_MTM_SECURITIZATION kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;
    
/*- haircut -*/
    %get_haircut;
    
/*- get effective maturity: for guarantee-like protection -*/
    %get_m;
    %check_m;
    
    _VALUE_ = VALUE_AMT;

    OUTV_FACE_VALUE = ISSUE_FACE_VALUE_AMT;
    
    /*- use user supplied EAD if supplied -*/
    if not missing(EAD_AMT) then
       OUTV_EAD = EAD_AMT/HOLDING;
    else 
       OUTV_EAD = _VALUE_;
   
endmethod;

/*- mark-to-market pricing method for credit derivatives as protection -*/
method PM_MTM_DERIVATIVE kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;
    
/*- haircut -*/
    %get_haircut(SCALE_ONLY=Y);
    
    _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
    OUTV_CRM = _VALUE_;

endmethod;

/*- mark-to-market pricing method for guarantee as protection -*/
method PM_MTM_GUARANTEE kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;
   
/*- haircut -*/
    %get_haircut(SCALE_ONLY=Y);

    _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
    OUTV_CRM = _VALUE_;

endmethod;

/*- mark-to-market pricing method for insurance as protection -*/
method PM_MTM_INSURANCE kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;
    
/*- haircut -*/
    %get_haircut 
   
    
    _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;

    OUTV_CRM = _VALUE_;

endmethod;

/*- mark-to-market pricing method for physical collateral -*/
method PM_MTM_PHYSICAL kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;
    
/*- haircut is not needed for physical collateral, but macro is retained -*/
/*- for customization purposes -*/
    /*- %get_haircut(SCALE_ONLY=Y); -*/
        
    if CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' and CRM_SUBTYPE = 'RRE' then do ; 
       if not missing (Mortgage_Lending_Value_Amt) then  
          _VALUE_ = 0.8 * Mortgage_Lending_Value_Amt ; 
       else  _VALUE_ = 0.8 * Value_Amt ; 
    end ;   
    
    else if CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' and CRM_SUBTYPE = 'CRE' then do ; 
       
       if not missing (Mortgage_Lending_Value_Amt) then  
          _VALUE_ = 0.6 * Mortgage_Lending_Value_Amt ; 
       else  _VALUE_ = 0.5 * Value_Amt ;     
    end ;      
    
    else _VALUE_ = PROTECTION_VALUE_AMT/HOLDING;
    
    OUTV_CRM = _VALUE_;

endmethod;

/*- method converting exposure-specific provisions into amounts in reporting currency -*/
method PM_MTM_EXPOSURE_PROVISION desc='Exposure-Specific Provision Pricing' kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;
    
    _VALUE_ = 0;
    OUTV_EXPOSURE_PROVISION = PROVISION_AMT;
    if PROVISION_TYPE = 'PARTIAL_WRITE_OFF' then do ; 
              OUTV_EXPOSURE_WRITEOFF = PROVISION_AMT;
    end; 

endmethod;

/*- method converting general provisions into amounts in reporting currency -*/
method PM_MTM_GENERAL_PROVISION desc='General Provision Pricing' kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;

    _VALUE_ = 0;
    OUTV_GENERAL_PROVISION = GENERAL_PROVISION;

endmethod;

/*- method converting facility groups into reporting currency -*/
method PM_MTM_FACILITY_GROUP desc='Group Facility Pricing' kind=price;

/*- initialization -*/
    %initialize_mtm_outvars;

    _VALUE_ =  LIMIT_AMT;
    OUTV_FACE_VALUE = LIMIT_AMT;
endmethod;

/*- method to test trading book counterparty risk - very elementary -*/
method PM_MTM_TEST_CCR kind = price;

/*- initialization -*/
    %initialize_mtm_outvars;

/*- haircut selection differs for repurchase style instruments -*/
if (STD_PRODUCT_CLASS = 'REPO_STYLE' or IRB_PRODUCT_CLASS = 'REPO_STYLE') then do ;
    %get_haircut(REPO_EXP=Y)
end ;
else do ;
    %get_haircut 
end ;    
    
/*- add-on -*/
    %get_addon_crd4;
    
    if not missing(EAD_AMT) then do;
       OUTV_EAD = EAD_AMT;
       if not missing(VALUE_AMT) then _VALUE_=VALUE_AMT;
       else _VALUE_= EAD_AMT;
    end;
    else
       _VALUE_ = RISKFACTOR.UNDERLYING_RISK_FACTOR_ID * FACE_VALUE_AMT;

/*- for master netting treatment of repo-style transactions and market-related off-balance-sheet items -*/
    OUTV_FACE_VALUE = FACE_VALUE_AMT;

    %get_m;
    %check_m;
        
    if PE_APPROACH = 'CE' then
       OUTV_EAD = max(0, _VALUE_) + FACE_VALUE_AMT * OUTV_ADD_ON / 100;
    else if PE_APPROACH = 'OE' then
       OUTV_EAD = FACE_VALUE_AMT * OUTV_ADD_ON / 100; 
    else
       OUTV_EAD = _VALUE_; 

endmethod;


run;
%mend d00_pm_mtm_crd4;
