/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: enrich_crms_crd4.sas
 *
 * PURPOSE: Stores code for enriching credit risk mitigant data.
 *
 *   USAGE: %enrich_crms_crd4(posds, issueds, outposds, );
 *          posds    : (pos) input position data set
 *          issueds  : (pos) input issue data set
 *          outposds : (pos) output position data set
 *
 *   NOTES: 
 *
 *
 *****************************************************************************/

%macro enrich_crms_crd4(posds, issueds, outposds);

    %local i MAT_EXIST INC_EXIST;

/*- enrich data with issue attributes and ratings -*/
    proc sort data = &posds out = &outposds ;
    by ISSUE_KEY;
    run;

    data &outposds;
    merge &outposds(in = a) RD_POS.issue_ratings &issueds;
    by ISSUE_KEY;
    if a;   
    run;    

/*- check if INCEPTION_DATE/MATURITY_DATE exists -*/
    %let MAT_EXIST = %varexist(&outposds, MATURITY_DATE);
    %let INC_EXIST = %varexist(&outposds, INCEPTION_DATE);

/*- enrich data with counterparty types -*/
    proc sort data = &outposds;
    by COUNTERPARTY_RK;
    run;

    data &outposds;
    merge &outposds(in=a) 
          RD_POS.counterparty_dim
          ;
    by COUNTERPARTY_RK;
    if a;
    run;

/*- enrich data with internal organization attributes -*/
    proc sort data = &outposds;
    by INTERNAL_ORG_RK;
    run;

    data &outposds(drop = %if &NUM_AGENCIES_ISSUE>0 %then %do; EXT_ISSUE_RATING_: EXT_ISSUE_MAP_RATING_:
                          CRM_ADJUSTED_FLG_: SHORT_TERM_ASSESS_FLG_: %end; 
             BASELI_CCY_FLAG BASELI_ST_TYPE BASELI_PRODUCT_CLASS BASELI_PRODUCT_SUBCLASS 
             BASELI_PRODUCT ELIGIBLE_DD_TREAT_OBLIGOR            
                 index= (INSTID));
    merge &outposds(in =a) RD_STAGE.internal_org_mart(in = b);
    by INTERNAL_ORG_RK;
    if a;
         
    length INSTTYPE CRM_TYPE CRM_SUBTYPE ELIGIBLE_CRM_TYPE HAIRCUT_SET HAIRCUT_TYPE 
              POSITION_CATEGORY BASELI_RW_CLASS $32. 
           BASELI_CCY_FLAG $16. CVA_STD_IMPLIED_RATING $20.
          EXT_ISSUE_MAP_RATING HAIRCUT_RATING_BUCKET STD_RW_BUCKET CVA_STD_CRM_BUCKET  
             HAIRCUT_RES_MAT_BAND BASELI_ST_TYPE $8. 
          LGD_BUCKET_SEN LGD_BUCKET_SUB COLL_LEVEL_BUCKET $10.;
           ;

    POSITION_CATEGORY = "CRM";
      
/*- check on currency -*/
    if  CURRENCY = '' then CURRENCY="&REPORT_CCY" ;

/*- if INCEPTION_DATE and MATURITY_DATE don't exist, set them -*/
/*- also be conservative if both are non-missing -*/
    %if &INC_EXIST = 0 %then %do;
        FORMAT INCEPTION_DATE NLDATE20.; 
        INCEPTION_DATE = CRM_EFFECTIVE_DATE;
    %end;
    %else %do;
       if INCEPTION_DATE = . then INCEPTION_DATE = CRM_EFFECTIVE_DATE;
       else if CRM_EFFECTIVE_DATE NE . then INCEPTION_DATE = max(INCEPTION_DATE, CRM_EFFECTIVE_DATE);
    %end;
    %if &MAT_EXIST = 0 %then %do;
        FORMAT MATURITY_DATE NLDATE20.; 
        MATURITY_DATE = CRM_EXPIRATION_DATE;
    %end;
    %else %do;
        if MATURITY_DATE = . then MATURITY_DATE = CRM_EXPIRATION_DATE;
        else if CRM_EXPIRATION_DATE NE . then MATURITY_DATE = min(MATURITY_DATE, CRM_EXPIRATION_DATE);        
    %end;

/*- get original and residual maturity -*/
    %get_maturity(INCEPTION_DATE, MATURITY_DATE);
    

/*- determine if it is a short-term claim -*/
    %get_shortterm_claim_flg_crd4;  

/*- get product class and subclass -*/
    %get_product_class;
    
/*- get LE product class -*/
    %get_le_exposure_type_crd4;
    
/*- get crm type and subtype -*/
    CRM_TYPE             = put(compress(CRM_TYPE_CD || CRM_SUBTYPE_CD), $MAP_CRM_TYPE.);
    CRM_SUBTYPE          = put(compress(CRM_TYPE_CD || CRM_SUBTYPE_CD), $MAP_CRM_SUBTYPE.);    
    ELIGIBLE_CRM_TYPE    = put(ELIGIBLE_CRM_TYPE_CD, $MAP_ELIGIBLE_CRM_TYPE.);

/*- funded credit derivatives such as credit linked notes can be treated as cash collaterals: CRR article 218 -*/
    if CRM_TYPE = 'CREDIT_DERIVATIVE' and CRM_SUBTYPE = 'FUNDED' then do ;
       STD_PRODUCT_CLASS = 'OTHER_ASSETS' ;
       STD_PRODUCT_SUBCLASS = 'CASH' ;
       IRB_PRODUCT_CLASS = 'OTHER' ;
       IRB_PRODUCT_SUBCLASS = 'CASH_GOLD_BULLION_EXP' ;
    end ;
    
/*- check double treatment eligibility -*/
    ELIGIBLE_DD_TREAT_CRM = 'N';
    if upcase("&DOUBLE_DEFAULT_TREATMENT")='YES' then do;
       ELIGIBLE_DD_TREAT_CRM= put(compress(CRM_TYPE_CD || CRM_SUBTYPE_CD), $MAP_ELIGIBLE_DD_TREAT_CRM.);
       if ELIGIBLE_DD_TREAT_GUARANTOR NE 'Y' then
          ELIGIBLE_DD_TREAT_CRM = 'N';
    end;

/*- get min LGD keys -*/
    LGD_BUCKET_SEN = put(compress('SEN' || CRM_TYPE_CD) , $MAP_MIN_LGD_BUCKET_IRB_COLL.);
    LGD_BUCKET_SUB = put(compress('SUB' || CRM_TYPE_CD) , $MAP_MIN_LGD_BUCKET_IRB_COLL.);

/*- get key for min/over collateralization levels -*/
    COLL_LEVEL_BUCKET = put(compress(CRM_TYPE_CD), $MAP_OVER_LEVEL_BUCKET_IRB_COLL.);

/*- get instrument type for MtM pricing -*/
    %get_mtm_insttype;
   
/*- get mapped issue rating: either from second best or from specified rating agency -*/
    %get_mapped_ratings(RATING_TYPE=ISSUE);
    
/*- get risk weight buckets -*/
    %get_std_rw_buckets_crm_crd4;  
   
/*- get Basel I risk weight mapping -*/
    %get_basel_i_rw;

/*- get the weight mapping for index CDS used to hedge -*/
/*- in the CVA standardized approach -*/
    %get_cva_crm_wgt_buckets_crd4 ;  

/*- get haircut info -*/
    %get_haircut_info;
   
    COLLATERAL = 1;
        
    run;
    
/* Redirect the rows which doesn't have valid Instrument Type */     
   data &outposds  invalid_insttypes ;
        set &outposds; 
        if INVALID_INSTTYPE_FLG eq 1 then output invalid_insttypes ;
        else output 
            &outposds; 
       run ;  

/*- append invalid Instrument Type data set -*/
    %if %attrn(invalid_insttypes, NOBS) GT 0 %then %do;
       proc append base=RD_POS.all_crm_invalid_insttypes data=invalid_insttypes force nowarn;
       run;
    %end;


%mend enrich_crms_crd4;
