/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: enrich_exposures_crd4.sas
 *
 * PURPOSE: Stores code for enriching exposures (except for facilities).
 *
 *   USAGE: %enrich_exposures_crd4(posds, issueds, outposds);
 *          posds    : (pos) input position data set
 *          issueds  : (pos) input issue data set
 *          outposds : (pos) output position data set
 *
 *   NOTES: 
 *
 *
 *****************************************************************************/

%macro enrich_exposures_crd4(posds, issueds, outposds, check_trading);

    %local DAYS_EXIST RESET_EXIST EQUITY_EXIST REPO_FLG SEGMENT_EXIST;
   
/*- 1: enrich data with issue attributes and ratings -*/
/*- preprocessing: get all the novation instid -*/
    %if "&CHECK_TRADING" = "Y" %then %do;
        proc sql;
        create table &outposds as select *, 
        (case when INSTID in (select NOVATION_INSTID from RD_STAGE.netting_set_mart) then 1
        else 0 end) as NOVATION_FLG
        from &posds
        order by ISSUE_KEY;
        quit;
    %end;
    %else %do;
    proc sort data=&posds out=&outposds;
    by ISSUE_KEY;
    run;
    %end;
    
/*- update issuer counterparty class for exposures like repo-style transactions -*/
    %let ISSUE_CPTY_EXIST=%varexist(&issueds, ISSUER_COUNTERPARTY_RK);
    %if &ISSUE_CPTY_EXIST %then %do;
        data issue_cpty;
        set &issueds (where=(not missing(ISSUER_COUNTERPARTY_RK)));
        run;
        
        proc sort data=issue_cpty;
        by ISSUER_COUNTERPARTY_RK;
        run;
    
        data issue_cpty (keep=ISSUE_KEY COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS index=(ISSUE_KEY)
                         rename=(COUNTERPARTY_CLASS=ISSUER_CPTY_CLASS COUNTERPARTY_SUBCLASS=ISSUER_CPTY_SUBCLASS));
        merge issue_cpty (rename=(ISSUER_COUNTERPARTY_RK=COUNTERPARTY_RK) in=a) 
              rd_pos.counterparty_dim;
        by COUNTERPARTY_RK;
        if a;
        run;
    %end;
    
    data &outposds;
    merge &outposds(in = a) RD_POS.issue_ratings RD_POS.issue_int_ratings &issueds 
          %if &ISSUE_CPTY_EXIST %then issue_cpty; ;
    by ISSUE_KEY;
    if a;
    run;    

  
/*- update ccp_ counterparty class for ccp related trade exposures -*/
    %let CCP_CPTY_EXIST=%varexist(&posds, CCP_COUNTERPARTY_RK);
    %if &CCP_CPTY_EXIST %then %do;
        data ccp_cpty (drop=COUNTERPARTY_RK);
        set &posds (where=(not missing(CCP_COUNTERPARTY_RK)));
        run;

        proc sort data=ccp_cpty;
        by CCP_COUNTERPARTY_RK;
        run;

        data ccp_cpty (keep=INSTID COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS index=(INSTID)
                          rename=(COUNTERPARTY_CLASS=CCP_CPTY_CLASS COUNTERPARTY_SUBCLASS=CCP_CPTY_SUBCLASS));
        merge ccp_cpty (rename=(CCP_COUNTERPARTY_RK=COUNTERPARTY_RK) in=a) 
               rd_pos.counterparty_dim;
        by COUNTERPARTY_RK;
        if a;
        run;
    %end;
    
    %if &CCP_CPTY_EXIST %then %do ;
        proc sort data= &outposds;
          by INSTID ;
        run ;

        data &outposds;
           merge &outposds(in = a)
                 ccp_cpty; 
           by INSTID;
           if a;
        run; 
    %end ;   
    

/*- check if DAYS_PAST_DUE_NO available -*/
    %let DAYS_EXIST = %varexist(&outposds, DAYS_PAST_DUE_NO);

/*- check if NEXT_CONTRACT_RESET_DATE available: for add-on under current exposure method -*/
    %let RESET_EXIST = %varexist(&outposds, NEXT_CONTRACT_RESET_DATE);

/*- check if EQUITY_POSITION_TYPE_CD exists -*/
    %let EQUITY_EXIST = %varexist(&outposds, EQUITY_POSITION_TYPE_CD);

/*- check if SEGMENT exists for Retail exposures-*/
    %let SEGMENT_EXIST = %varexist(&outposds, SEGMENT_ID);
    
/*- check if REPO_TRANSACTION_FLG exists -*/
    %let REPO_FLG = %varexist(&outposds, REPO_TRANSACTION_FLG);   

    /*-Check is the retail data is available at ind. exposure level and configuration is USFinal
    then do the merging on segment_id on segment_mart table*/
    %if &SEGMENT_EXIST %then %do;
        %if &RETAIL_DATA_LOAD_USFINAL = YES %then %do;
            %if &RETAIL_EXP_IND_LOAD_USFINAL = YES %then %do;
            proc sort data= &outposds; 
            by SEGMENT_ID;
            run;
            data &outposds;
            merge &outposds(in = a) RD_STAGE.SEGMENT_MART;
            by SEGMENT_ID;
            if a;
            run;
            %end;
        %end;
    %end;
    

/*- 2: enrich data with counterparty dimensional data -*/
    proc sort data = &outposds;
    by COUNTERPARTY_RK;
    run;

    data &outposds;
    merge &outposds(in=a) 
          RD_POS.counterparty_dim
          ;
    by COUNTERPARTY_RK;
    if a;
    run;
    
  
/*- 3: enrich data with internal organization attributes -*/
    proc sort data = &outposds;
    by INTERNAL_ORG_RK;
    run;

    data &outposds(drop = %if &NUM_AGENCIES_ISSUE>0 %then %do; EXT_ISSUE_RATING_: EXT_ISSUE_MAP_RATING_:
                   CRM_ADJUSTED_FLG_: SHORT_TERM_ASSESS_FLG_: %end; LOCATION ORIGINAL_APPROACH REASON
                   INVALID_COMPONENT_FLG CPTY_PD_FLG PD_FLG LGD_FLG INT_ISSUE_RATING
                   BASELI_CCY_FLAG BASELI_ST_TYPE BASELI_PRODUCT_CLASS BASELI_PRODUCT_SUBCLASS 
                   BASELI_PRODUCT ELIGIBLE_DD_TREAT_GUARANTOR INVALID_APPROACH_FLG 
                   %if "&CHECK_TRADING" = "Y" %then %do; NOVATION_FLG %end;
                  )
         &outposds._sec(drop = INVALID_APPROACH_FLG LOCATION ORIGINAL_APPROACH REASON INVALID_COMPONENT_FLG
                               CPTY_PD_FLG PD_FLG LGD_FLG ELIGIBLE_DD_TREAT_GUARANTOR 
                               %if "&CHECK_TRADING" = "Y" %then %do; NOVATION_FLG %end;
                  )
         %if "&CHECK_TRADING" = "Y" %then %do; RD_POS.exposure_trading_book
                  (drop = %if &NUM_AGENCIES_ISSUE>0 %then %do; EXT_ISSUE_RATING_: EXT_ISSUE_MAP_RATING_:
                   CRM_ADJUSTED_FLG_: SHORT_TERM_ASSESS_FLG_: %end; LOCATION ORIGINAL_APPROACH REASON
                   INVALID_COMPONENT_FLG CPTY_PD_FLG PD_FLG LGD_FLG INT_ISSUE_RATING
                   BASELI_CCY_FLAG BASELI_ST_TYPE BASELI_PRODUCT_CLASS BASELI_PRODUCT_SUBCLASS 
                   BASELI_PRODUCT ELIGIBLE_DD_TREAT_GUARANTOR INVALID_APPROACH_FLG NOVATION_FLG
                   index=(NETTING_SET_RK)
                  )
         %end;
         invalid_approach(keep = INSTID COUNTERPARTY_CLASS STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS
                           APPROACH_CLASS APPROACH )
         invalid_components (keep=INSTID LOCATION ORIGINAL_APPROACH APPROACH REASON CPTY_PD_MODEL_ID CPTY_PD_RATING_GRADE PD_MODEL_ID 
                                  PD_MODEL_RATING_GRADE LGD_MODEL_ID LGD_MODEL_RATING_GRADE PD_PCT LGD_PCT CPTY_PD_PCT)
         ; 

    merge &outposds(in =a) RD_STAGE.internal_org_mart(in = b);
    by INTERNAL_ORG_RK;
    if a;

    length STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS
           APPROACH_CLASS INSTTYPE OFF_BALANCE_SHEET_TYPE HAIRCUT_TYPE ADDON_TYPE
           HAIRCUT_SET POSITION_CATEGORY LOCATION REASON BASELI_RW_CLASS $32. 
           ORIGINAL_APPROACH APPROACH BASELI_CCY_FLAG $16.;
    length PE_APPROACH EXT_ISSUE_MAP_RATING INT_ISSUE_MAP_RATING 
           HAIRCUT_RATING_BUCKET STD_RW_BUCKET IRB_RW_BUCKET 
           HAIRCUT_RES_MAT_BAND ADDON_MAT_BAND 
           BASELI_ST_TYPE $8.
           EXP_NOT_OWNED_FLG 8.;
          
/*- check on currency -*/
    if  CURRENCY = '' then CURRENCY="&REPORT_CCY" ;

/*- get seniority flag -*/
    %get_seniority_flg;

    %if &RESET_EXIST = 0 %then %do;
        NEXT_CONTRACT_RESET_DATE = .;
    %end;
    
/*- flag exposures underlying securitization pools -*/ 
    %get_securitized_exposure_flg; 
    
/*- flag for covering securitization pool -*/  
    if COVER_SECURITIZATION_POOL_RK NE . and APPROACH NE 'BASEL_I' then do;
          SECURITIZATION_EXPOSURE_FLG = 1;
          SEC_UNDERLYING_EXP_FLG = 0; 
          SECURITIZATION_POOL_RK = COVER_SECURITIZATION_POOL_RK ; 
    end;
    else do;
       SECURITIZATION_EXPOSURE_FLG = 0;  
    end ;

/*- flag indicating the exposure is not owned by the reporting institution -*/            
    EXP_NOT_OWNED_FLG = put(FINANCIAL_BOOK_RK, MAP_EXP_NOT_OWNED_FLG.) ;
    
/*- get original and residual maturity -*/
/*- If the transaction is a repo style transaction -*/
/*- then the maturity of the repo is what is used, not -*/
/*- that of the underlying instrument. This is saved to  -*/
/*- a new variable and carried through to the MTM method -*/
/*- The residual maturity of the exposure is also calculated -*/
%if NEXT_CONTRACT_RESET_DATE ne . and IRB_PRODUCT_CLASS = 'DERIVATIVE' %then %do ;
    %get_maturity(INCEPTION_DATE, NEXT_CONTRACT_RESET_DATE); 

%end ; 
%else %if &REPO_FLG %then %do ;
 if REPO_TRANSACTION_FLG then do; 
    
        %get_maturity(REPO_INCEPTION_DT,REPO_MATURITY_DT) ;
        RESIDUAL_MAT_REPO_CONTRACT = RESIDUAL_MATURITY ;     
        ORIG_MAT_REPO = ORIGINAL_MATURITY; 
        
        %get_maturity(INCEPTION_DATE, MATURITY_DATE); 
        ORIGINAL_MATURITY = ORIG_MAT_REPO; 
  
  end; 
  else do; 
    %get_maturity(INCEPTION_DATE, MATURITY_DATE); 
  end; 
%end ;
%else %do ; 
   
    %get_maturity(INCEPTION_DATE, MATURITY_DATE); 

%end ; 

/*- determine if it is considered past due: more than &DAYS_PAST_DUE_THRESHOLD days -*/
    %if &DAYS_EXIST %then %do;
        PAST_DUE_CLAIM_FLG = (DAYS_PAST_DUE_NO > &DAYS_PAST_DUE_THRESHOLD);
    %end;

/*- get product class and subclass -*/
    %get_product_class;
    
/*- get LE product class -*/
    %get_le_exposure_type_crd4;
    
/*- apply repo transaction flag if it is set -*/
    %if &REPO_FLG %then %do;
        if REPO_TRANSACTION_FLG then do;
           STD_PRODUCT_CLASS = "REPO_STYLE";
           STD_PRODUCT_SUBCLASS = "";
           IRB_PRODUCT_CLASS = "REPO_STYLE";
           IRB_PRODUCT_SUBCLASS = "";
        end;
    %end;

/*- get regulatory approach -*/
    %get_regulatory_approach;
    
/*- check proper approach assignment -*/
    %check_asset_class_approach;

    if INVALID_APPROACH_FLG EQ 1 then 
       output invalid_approach;
/*- process the following only for valid approaches -*/
    else do; 
    
/*- check necessary component for IRB aproaches -*/    
       %check_component;
            
       LOCATION="&outposds";
       if INVALID_COMPONENT_FLG = 1 then
          output invalid_components;
          
/*- determine if it is a short-term claim: original maturity of 3 months or less -*/
       %get_shortterm_claim_flg_crd4;  

/*- get instrument type for MtM pricing -*/
       %get_mtm_insttype;

/*- get off-balance-sheet type -*/
       %get_obsi_type;
       
       
/*- get LE off-balance-sheet type -*/
    %get_le_obsi_type_crd4;

       
/*- get mapped issue rating: either from second best or from specified rating agency -*/
       %get_mapped_ratings(RATING_TYPE=ISSUE);
       INT_ISSUE_MAP_RATING = put(INT_ISSUE_RATING, $INT.);

/*- get pd floor and lgd buckets for equities -*/
       %if &EQUITY_EXIST %then %do;
       
          COUNTERPARTY_RELATION_TYPE = put(COUNTERPARTY_RELATION_TYPE_CD, $MAP_COUNTERPARTY_RELATION_TYPE.);
          if APPROACH in ('IRB_PD_LGD', 'IRB_INT_MODELS') then do;
            PD_MODEL_FLOOR_GRADE =  put(compress(EQUITY_POSITION_TYPE_CD || REGULATORY_PRODUCT_CD), $MAP_PD_BUCKET_IRB_EQUITY.);
        /* CRR Article 161 (1) : 0.09 % for exchange traded equity exposures where the investment is part of a 
           long-term customer relationship;  */    
           if IRB_PRODUCT_SUBCLASS = 'EQUITY_PUBLIC' and COUNTERPARTY_RELATION_TYPE = 'LONGTERM' then 
               PD_MODEL_FLOOR_GRADE = 'BUCKET1' ; /*BUCKET1 corresponds to 0.09% */
           
            /*- No user estimate of LGD is allowed for equities under PD_LGD approach, so no -*/
            /*- floor is needed. This is used to input the regulatory value -*/
            LGD_MODEL_RATING_GRADE = put(compress(EQUITY_POSITION_TYPE_CD || REGULATORY_PRODUCT_CD), $MAP_LGD_BUCKET_IRB_EQUITY.);
          end;
          
       %end;

/*- get risk weight buckets -*/
       %get_std_rw_buckets_crd4; 
       %get_irb_rw_buckets(EQUITY_EXIST=&EQUITY_EXIST);   
       
/*- get Basel I risk weight mapping -*/
       %get_basel_i_rw;
    
/*- get haircut info -*/
       %get_haircut_info;

/*- get add-on info -*/
       %get_addon_info(RESET_EXIST=&RESET_EXIST);    

/*- get position type -*/
       %get_position_type;
       
/*- update repo-style eligible carve out treatment flag -*/
       if %upcase("&HAIRCUT_CARVE_OUT_REPO") = "YES" and ELIGIBLE_REPO_CARVE_OUT_FLG = 1 and 
          CORE_MARKET_PARTICIPANT_FLG = 1 then 
          ELIGIBLE_REPO_CARVE_OUT_FLG = 1;
       else
          ELIGIBLE_REPO_CARVE_OUT_FLG = 0;

       COLLATERAL = 0;

  
/*- check for trading book items -*/
       TRADING_BOOK_FLG = put(FINANCIAL_BOOK_RK, MAP_TRADING_BOOK_FLG.);
       
/*- direct to the output data set -*/
       %if "&CHECK_TRADING" = "Y" %then %do;
           if TRADING_BOOK_FLG = 1 and NOVATION_FLG NE 1 then do;
              CCR_APPROACH = put(trim(&APPROACH_CCS), $MAP_CCR_APPROACH.);
              if CCR_APPROACH in ("IMM", "STD") then 
                 output RD_POS.exposure_trading_book;
              else do;
                 CCR_APPROACH = "CEM";
                 output &outposds;  
              end;
           end;
           else 
       %end;
           if SEC_UNDERLYING_EXP_FLG EQ 1 then 
              output &outposds._sec;
           else 
              output &outposds;
    end;
        
    run;

/* Redirect the rows which doesn't have valid Instrument Type */     
   data &outposds  invalid_insttypes ;
        set &outposds; 
        if INVALID_INSTTYPE_FLG eq 1 then output invalid_insttypes ;
        else output 
            &outposds; 
       run ;  

   data &outposds._sec invalid_insttypes_sec ; 
     set &outposds._sec; 
       if INVALID_INSTTYPE_FLG eq 1 then output invalid_insttypes_sec ;
        else output 
           &outposds._sec; 
    run ;  
    
/*- check if there is any non-CEM trading book in the system -*/
    %if "&CHECK_TRADING" = "Y" %then %do;
        %let N_TRADING_BOOK_EXP = %attrn(RD_POS.exposure_trading_book, NOBS);
    %end;
    
/*- append invalid approach data set -*/
    %if %attrn(invalid_approach, NOBS) GT 0 %then %do;
       proc append base=RD_POS.invalid_approach data=invalid_approach ;
       run;
    %end;
    
/*- append invalid component data set -*/
    %if %attrn(invalid_components, NOBS) GT 0 %then %do;
       proc append base=RD_POS.all_exp_with_invalid_components data=invalid_components force ;
       run;
    %end;

/*- append invalid Instrument Type data set -*/
    %if %attrn(invalid_insttypes, NOBS) GT 0 %then %do;
       proc append base=RD_POS.all_exp_invalid_insttypes data=invalid_insttypes force nowarn;
       run;
    %end;

/*- append invalid Instrument sec Type data set -*/
    %if %attrn(invalid_insttypes_sec, NOBS) GT 0 %then %do;
       proc append base=RD_POS.all_exp_invalid_insttypes_sec data=invalid_insttypes_sec force nowarn;
       run;
    %end;

%mend enrich_exposures_crd4;
