/*****************************************************************************
 * Copyright (c) 2013 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: get_cva_risk_amts_crd4.sas
 *
 * PURPOSE: Code to calculate CVA risk amounts for CVA under standardized     
 *          approach.
 *                   
 *   USAGE: %get_cva_risk_amts_crd4
 *               CRMAPP  : CRM approach applied (OPTIMAL or BY_RANK)
 *               ALTCALC : If alternative approach should be applied to OEM exposures (Y/N)
 *
 *           
 *   NOTES: 
 *          
 *****************************************************************************/
%macro get_cva_risk_amts_crd4(CRMAPP=,ALTCALC=);

/* Create a table with all the exposures subject to the CVA calculation. Also, bring in the */
/* adjusted EAD amount. This value comes from from the ALLMITIGATION table under BY_RANK */
/* and from CRM_ALLOCATION_OPTIMAL under OPTIMAL */
%let RPTPATH = %sysget(CRISKMRES)&pathsep.&project_set;
libname _intds "&RPTPATH";
libname _regds "&REGPATH" ; 

%if &CRMAPP = BY_RANK %then %do ; 
   proc sql noprint ; 
   create table _cva_exp_list_ as
       select a.*, b.OUTV_EAD_ADJ
           from (select *, COUNTERPARTY_ID as EXP_CPTY_ID
               from _regds.INSTVALS
                    where index(INSTID,"MNA_OBSI") ne 0  
                    or (STD_PRODUCT_CLASS in ('DERIVATIVE','REPO') 
                    or IRB_PRODUCT_CLASS in ('DERIVATIVE','REPO'))) as a 
              left join _detail.allmitigation as b
           on a.INSTID = b.EXPOSUREID
           where missing(CRM_TYPE) ; 
   quit ;
%end ; 
%else %if &CRMAPP = OPTIMAL %then %do ; 
   proc sql noprint ; 
   create table _cva_exp_list_ as
       select a.*, b.EAD_ADJ as OUTV_EAD_ADJ
           from (select *, COUNTERPARTY_ID as EXP_CPTY_ID
               from _regds.INSTVALS
                    where index(INSTID,"MNA_OBSI") ne 0  
                    or (STD_PRODUCT_CLASS in ('DERIVATIVE','REPO') 
                    or IRB_PRODUCT_CLASS in ('DERIVATIVE','REPO'))) as a 
              left join _intds.crm_allocation_optimal as b
           on a.INSTID = b.EXP_ID 
           where missing(a.CRM_TYPE) ; 
   quit ;
%end ; 

/*- If no collateral was used to adjust EAD, then use EAD -*/
/*- as the value. Otherwise, use OUTV_EAD_ADJ -*/
/*- Also, output any exposures for alternative approach -*/
/*- to a separate data set. This data step also separates the  -*/
/*- EAD into two new variables for reporting purposes -*/
data _cva_exp_list_ _cva_alt_input_; 
   set _cva_exp_list_ ; 
   
   if not missing(OUTV_EAD_ADJ) then
      CVA_EAD = OUTV_EAD_ADJ ; 
   else CVA_EAD = EAD ;

   if STD_PRODUCT_CLASS = 'REPO' or IRB_PRODUCT_CLASS = 'REPO' then
      CVA_EAD_SFT = CVA_EAD ; 
   else 
      CVA_EAD_OTC = CVA_EAD ; 
   
   _ccr_apprch_ = put(APPROACH_CLASS,$MAP_CCR_APPROACH.) ;

   %if &ALTCALC = Y %then %do ; 
      if _ccr_apprch_ not in ('CEM','IMM') then
         output _cva_alt_input_; 
      else output _cva_exp_list_ ; 
   %end ; 
   %else %do ; 
      output _cva_exp_list_ ;
   %end ; 

drop OUTV_EAD_ADJ EAD _ccr_apprch_ ;
run ; 

/*Link the CRM associated with these exposures */
proc sql noprint ; 
create table _cva_exp_crm_link_ as
    select a.*, b.*
        from _cva_exp_list_ as a left join _regds.full_exposure_crm_link as b
        on a.INSTID = b.EXP_ID ; 
quit ; 

/*Join the information about the CRM to the exposure information. This final table*/
/*will be split into one for exposures and one for CRM. Now, however, they */
/*are sharing the exposure counterparty.*/
proc sql noprint ; 
create table _cva_exp_crm_ as
    select a.*, b.PROTECTION_VALUE_AMT as CRM_PROTECT_AMT, b.ORIGINAL_MATURITY as CRM_ORIG_MAT, b.CRM_TYPE as HEDGE_INST_TYPE,
           b.CRM_SUBTYPE as HEDGE_INST_SUBTYPE
        from _cva_exp_crm_link_ as a left join _regds.INSTVALS as b
        on a.CRM_ID = b.INSTID ; 
quit ; 


/* Create a table with the exposures. These will be summed by counterparty. */
/* Use a sort and a data step to remove duplicate exposures */
proc sort data = _cva_exp_crm_ out = _cva_exp_crm_ nodup ;
by INSTID ;
run ;

data _cva_exp_crm_ ;
    set _cva_exp_crm_ ;
    by INSTID ; 
    if first.INSTID then
        output ; 
run ;

proc sql noprint ; 
create table _cva_exp_detail_ as
    select INSTID, CVA_EAD, CVA_EAD_OTC, CVA_EAD_SFT, OUTV_CVA_STD_WGT,  sum(CVA_EAD*CVA_STD_MAT_EFF) as _wgt_ead_,
           (1-exp(-0.05*CVA_STD_MAT_EFF))/(0.05*CVA_STD_MAT_EFF) as _ead_disc_, 
           EXP_CPTY_ID
    from _cva_exp_crm_ 
    group by EXP_CPTY_ID ;
quit ; 

data _cva_exp_ (keep = EXP_CPTY_ID CVA_EAD CVA_EAD_OTC CVA_EAD_SFT OUTV_CVA_STD_WGT _final_ead_val_); 
    set _cva_exp_detail_; 
    by EXP_CPTY_ID ;
    if first.EXP_CPTY_ID then do ; 
        _final_ead_val_ = _wgt_ead_*_ead_disc_ ; 
        output ; 
    end ; 
run ;
  
/* Use the above table to find CRM (non-index) associated with the exposures. The data step is used to */
/* remove duplicates */
proc sql noprint ; 
    create table _cva_crm_detail_ as
        select CRM_ID, INSTID, EXP_CPTY_ID, CRM_ORIG_MAT, CRM_PROTECT_AMT, HEDGE_INST_TYPE, HEDGE_INST_SUBTYPE, 
                sum(CRM_PROTECT_AMT*CRM_ORIG_MAT) as _wgt_crm_, 
                (1-exp(-0.05*CRM_ORIG_MAT))/(0.05*CRM_ORIG_MAT) as _crm_disc_
        from _cva_exp_crm_
        where not missing(HEDGE_INST_TYPE) and HEDGE_INST_SUBTYPE ne 'INDEX'
    group by EXP_CPTY_ID; 
quit ;

data _cva_crm_ (keep = EXP_CPTY_ID _final_crm_val_ CRM_PROTECT_AMT ); 
    set _cva_crm_detail_; 
    by EXP_CPTY_ID ;
    if first.EXP_CPTY_ID then do ; 
        _final_crm_val_ = _wgt_crm_*_crm_disc_ ; 
        output ; 
    end ; 
run ;

/*Use the above table to find CRM (index) associated with the exposures. This can be summed here */
/*since the total is used in the final calculation*/
proc sql noprint ; 
    select sum(((1-exp(-0.05*ORIGINAL_MATURITY))/(0.05*ORIGINAL_MATURITY))*(OUTV_CVA_STD_WGT_CRM/100)*ORIGINAL_MATURITY*PROTECTION_VALUE_AMT),
           sum(PROTECTION_VALUE_AMT)
           into : _index_cds_value_, : _index_cds_notional_ 
    from _regds.INSTVALS
    where INSTID in (
                     select distinct CRM_ID from _regds.full_exposure_crm_link 
                        where EXP_ID in (select distinct INSTID from _cva_exp_detail_))
        and CRM_SUBTYPE = 'INDEX' ; 
quit ;

/*Calculate the final value for standardized approach */
data _cva_calc_final_ ; 
    merge _cva_exp_ _cva_crm_ end = last;
    by EXP_CPTY_ID ;

    retain _exp_amt_ _exp_amt_sft_ _exp_amt_otc_ _wgt_net_exp_hedge_ _wgt_net_exp_hedge_sq_ _cds_notional_;
    
    _wgt_net_exp_hedge_ = sum(_wgt_net_exp_hedge_,0.5*(OUTV_CVA_STD_WGT/100)*sum(_final_ead_val_,-_final_crm_val_)) ;
    _wgt_net_exp_hedge_sq_ = sum(_wgt_net_exp_hedge_sq_,0.75*((OUTV_CVA_STD_WGT/100)**2)*(sum(_final_ead_val_,-_final_crm_val_)**2)) ;
    _cds_notional_ = sum(_cds_notional_,CRM_PROTECT_AMT) ;
    _exp_amt_ = sum(_exp_amt_,CVA_EAD) ; 
    _exp_amt_otc_ = sum(_exp_amt_otc_,CVA_EAD_OTC) ; 
    _exp_amt_sft_ = sum(_exp_amt_sft_,CVA_EAD_SFT) ; 

    if last then do ;
        CVA_CHARGE_AMT=2.33*sqrt(&CVA_RISK_HORIZON)*sqrt(sum(sum(_wgt_net_exp_hedge_,-&_index_cds_value_)**2,_wgt_net_exp_hedge_sq_));
        CVA_CALCULATION_METHOD_TYPE = 'STD' ; 
        CVA_COUNTERPARTY_COUNT = _N_ ; 
        CVA_INDEX_NOTIONAL_AMT = &_index_cds_notional_ ;
        CVA_CDS_NOTIONAL_AMT = _cds_notional_ ; 
        EAD_ADJ = _exp_amt_ ; 
        CVA_EAD_ADJ_OTC = _exp_amt_otc_ ; 
        CVA_EAD_ADJ_SFT = _exp_amt_sft_ ; 
        output ;
    end ;

keep CVA_CHARGE_AMT CVA_CALCULATION_METHOD_TYPE CVA_COUNTERPARTY_COUNT 
     CVA_INDEX_NOTIONAL_AMT CVA_CDS_NOTIONAL_AMT EAD_ADJ CVA_EAD_ADJ_OTC CVA_EAD_ADJ_SFT ;
run ;
 
/*- If applicable, produce the alternative measure for exposures utilizing -*/
/*- the OEM approach (CRR Article 385). Note that the data set is sorted -*/ 
/*- to remove any duplicates. -*/
%if %attrn(_cva_alt_input_, NOBS) ne 0 %then %do;

   proc sort data = _cva_alt_input_ nodup ; 
   by INSTID ; 
   run ;

   proc sql noprint ; 
   create table _cva_alt_res_ as 
      select 5*sum(((OUTV_RW_ISSUE/100)*CVA_EAD)/12.5) as CVA_CHARGE_AMT, 
             'ALT' as CVA_CALCULATION_METHOD_TYPE, 
             count(EXP_CPTY_ID) as CVA_COUNTERPARTY_COUNT,
             sum(CVA_EAD) as EAD_ADJ,
             sum(CVA_EAD_SFT) as CVA_EAD_ADJ_SFT,
             sum(CVA_EAD_OTC) as CVA_EAD_ADJ_OTC
         from _cva_alt_input_ ;
   quit ; 

   /*- Set the two sets of results to be appended to the detail table -*/
   data _cva_calc_final_ ; 
      set _cva_calc_final_ _cva_alt_res_ ; 
   run ;   
%end ;

%mend get_cva_risk_amts_crd4;
