/*****************************************************************************
 * Copyright (c) 2013 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: get_rw_adj_immprpty_crd4.sas
 *
 * PURPOSE: Code to break down the EXP_VALUE and RW_EXP secured by immovable 
 *          properties into fully secured and not fully secured parts 
 *          under standardized approach.
 *
 *   USAGE: %get_rw_adj_immprpty_crd4
 *               crmalloc: CRM allocation method applied (optimal or by_rank)
 *
 *   NOTES: 1. Code supports the cases where each exposure has no more than one
 *             immovable property as physical collateral. 
 *          2. Multiple exposures can share the same immovable property collateral. 
            3. Other CRMs are allowed.
 *          
 *****************************************************************************/
%macro get_rw_adj_immprpty_crd4(crmalloc=) ;

/*- get exposures secured by immovable properties under STD approach    -*/       
data exposure_crm_imp;
   set &&RESLIB1..crm_allocation_&crmalloc._detail(keep=EXP_ID EXPOSURE_RANK CCF CRM_ID CRM_TYPE CRM_VALUE 
                      EAD_PORTION EXP_VALUE RW_EXP RW_CRM STD_PRODUCT_CLASS CRM_STD_PRODUCT_CLASS APPROACH); 
                      
   if APPROACH = 'STD' and STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') 
   and CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' and CRM_STD_PRODUCT_CLASS = 'IMM_PROPERTY' ;
   if CCF = . then CCF = 100 ;
   
run  ; 

/*- define order of allocating immovable property crm to the original exposure  -*/
proc sort data=exposure_crm_imp;
   %if &crmalloc = by_rank %then %do ;
       by CRM_ID EXPOSURE_RANK descending CCF descending RW_EXP descending EXP_VALUE;
   %end ;
   %else %if &crmalloc = optimal %then %do ;
       by CRM_ID descending CCF descending RW_EXP descending EXP_VALUE;
   %end ;     
run;

/*- EXP_VALUE_SEC is the part of the original EXP_VALUE that is fully secured by the imm. property.    -*/
/*- EXP_VALUE_UNSEC is the part of original EXP_VALUE that is not fully secured by the imm. property.  -*/
/*- RW_EXP_IMP_ADJ is the adjusted risk weight of the exposure: secured part gets the CRM risk weight. -*/
/*- unsecured part keeps the original RW_EXP.                                                          -*/
data exposure_crm_imp ;
 set exposure_crm_imp ;
  by CRM_ID ;
  if first.CRM_ID then do ;
     CRM_VALUE_REM = CRM_VALUE ;
     EXP_VALUE_UNSEC = EXP_VALUE ; 
  end ;
  retain  CRM_VALUE_REM  ;

  EXP_VALUE_SEC = min(EXP_VALUE, CRM_VALUE_REM);
  RW_EXP_IMP_ADJ = RW_CRM ;
  EXP_VALUE_UNSEC = EXP_VALUE - EXP_VALUE_SEC ;
  CRM_VALUE_REM = MAX(0,CRM_VALUE_REM - EXP_VALUE_SEC) ;
run;

data exposure_crm_sec ;
 set exposure_crm_imp (keep = EXP_ID CRM_ID EXP_VALUE_SEC RW_EXP_IMP_ADJ
                     rename = (EXP_VALUE_SEC =EXP_VALUE_IMP_ADJ)) ;
run ;

data exposure_crm_unsec ;
 set exposure_crm_imp (keep = EXP_ID EXP_VALUE_UNSEC RW_EXP
                     rename = (EXP_VALUE_UNSEC =EXP_VALUE_IMP_ADJ RW_EXP = RW_EXP_IMP_ADJ)) ;
run ;

proc sort data=exposure_crm_sec;
  by EXP_ID CRM_ID ;
run ;

proc sort data=exposure_crm_unsec;
  by EXP_ID  ;
run ;

/*- dataset that contains the adjusted exposure value and risk weight -*/
data exposure_crm_imp_adj ;
 set exposure_crm_sec exposure_crm_unsec ;
  by EXP_ID;
run ;

proc sort data=exposure_crm_imp_adj;
  by EXP_ID CRM_ID ;
run ;

proc sort data = &&RESLIB1..crm_allocation_&crmalloc._detail ;
  by EXP_ID CRM_ID ;
run ;

/*- add EXP_VALUE_IMP_ADJ and RW_EXP_IMP_ADJ to the original dataset used for reporting analysis  -*/  
data &&RESLIB1..crm_allocation_&crmalloc._detail ;
  merge &&RESLIB1..crm_allocation_&crmalloc._detail(in=a) exposure_crm_imp_adj(in=b);
    by  EXP_ID CRM_ID ;
  if a;
  SPECIFIC_PROVISION_IMP_ADJ = EXP_VALUE_IMP_ADJ / EXP_VALUE * SPECIFIC_PROVISION ;
  SPECIFIC_WRITEOFF_IMP_ADJ = EXP_VALUE_IMP_ADJ / EXP_VALUE * SPECIFIC_WRITEOFF ; 
  %if &crmalloc = by_rank %then %do ;
      if not missing(CRM_ID) then _cost_ = _weight_ ;
      else _cost_ = RW_EXP ;
  %end ; 
  
  run;

proc sort data = &&RESLIB1..crm_allocation_&crmalloc._detail;
  by EXP_ID _cost_  ;
run ;

%if &crmalloc = by_rank %then %do ;
    proc sql ;
    alter table &&RESLIB1..crm_allocation_&crmalloc._detail
      drop _cost_ ;
 %end ; 
 
%mend get_rw_adj_immprpty_crd4 ;
