/*****************************************************************************
 * Copyright (c) 2014 by SAS Institute Inc., Cary, NC, USA.
 *
 *    NAME: r00_post_mtm_3_crd4.sas
 *
 * PURPOSE: Process exposures in default. 
 *          
 *          
 *
 *   USAGE: %r00_post_mtm_3_crd4
 *
 *   NOTES: 
 *         4/30/2013  
 *         Rewritten to process both passdue loans and other exposures in default 
 *         under STD approach for CRD-IV.
 *         5/21/2014
 *         Update to treat all exposures to a defaulted non-retail counterparty 
 *         as defaulted. 
 *          
 *****************************************************************************/
 
%macro r00_post_mtm_3_crd4;

    %if APPROACH = 'BASEL_I' %then %goto EXIT;

/*- 1: get all defaulted exposures under STD approach -*/
    /*- get the original defaulted exposures under STD approach -*/
    data _exp_in_def_orig_(keep=EXP_ID EAD STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS NETTING_SET_RK 
                           SPECIFIC_PROVISION COUNTERPARTY_CLASS COUNTERPARTY_RK);
        set &RESLIB1..all_exposures;
        where (PAST_DUE_CLAIM_FLG = 1 or IN_DEFAULT_FLG =1) and APPROACH = 'STD';
    run;

    /*- Identify defaulted counterparties:  -*/
    /*- For a non-retail counterparty, if one exposure is defaulted,    -*/
    /*- the counterparty is considered defaulted and all exposures      -*/
    /*- by the counterparty are considered defaulted: CRR article 178   -*/
    data _cpty_in_def_ (keep =COUNTERPARTY_RK COUNTERPARTY_CLASS);
        set _exp_in_def_orig_ ;
    run ;

    proc sort data= _cpty_in_def_ nodupkey; 
         by COUNTERPARTY_RK ;
    run ;

    proc sort data = &RESLIB1..all_exposures; 
         by COUNTERPARTY_RK ;
    run ;
    
    /*- update exposures in default to include all exposures to defaulted non-retail counterparites -*/
    data _exp_in_def_ ;
        merge &RESLIB1..all_exposures _cpty_in_def_ (in=b keep=COUNTERPARTY_RK) ; 
        by COUNTERPARTY_RK ;
        if b ;
        if COUNTERPARTY_CLASS ne 'RETAIL' then IN_DEFAULT_FLG = 1 ;
    run ;

    proc sort data=&RESLIB1..all_exposures ;
         by EXP_ID ;
    run;

    proc sort data=_exp_in_def_ ;
        by EXP_ID ;
    run; 

/*- 2: join get all defaulted exposures with CRMs -*/
    proc sql;
    /*- get defaulted-netting links -*/
    create table defaulted_netting_link as
    select exp.EXP_ID, CRM_ID, crm.ELIGIBLE_CRM_TYPE, CRM_VALUE
    from _exp_in_def_ as exp, &RESLIB1..crms_netting as crm
    where exp.NETTING_SET_RK = crm.NETTING_SET_RK 
    order by EXP_ID;

    /*- get defaulted-crm links -*/
    create table defaulted_crm_link as
    select exp.EXP_ID, crm.CRM_ID, crm.ELIGIBLE_CRM_TYPE, crm.CRM_VALUE 
    from _exp_in_def_ as exp
    inner join RD_POS.exposure_crm_link as link 
    on exp.EXP_ID = link.EXP_ID
    inner join &RESLIB1..all_crms as crm 
    on crm.CRM_ID = link.CRM_ID
    order by EXP_ID;
    quit;

    /*- all exp_in_default under STD with CRM links -*/
    data _exp_in_def_with_crm_;
        set defaulted_netting_link defaulted_crm_link;  
        by EXP_ID;
        
        if ELIGIBLE_CRM_TYPE in ('FINANCIAL_SIMPLE', 'FINANCIAL_COMPREHENSIVE', 'GUARANTEE_STD', 'CREDIT_DERIVATIVE', 'NETTING', 'IRB_COLLATERAL') then
            CRM_ELIGIBLE_STD_FLG = 1;
    
        else
            CRM_ELIGIBLE_STD_FLG = 0;
    run;

/*- 3: get total eligible STD CRM and other CRM -*/
    data _exp_in_def_crm_total_(keep = EXP_ID CRM_VALUE_STD CRM_VALUE_OTHER);
    set _exp_in_def_with_crm_;
    by EXP_ID;
    retain CRM_VALUE_STD CRM_VALUE_OTHER 0;
    if first.EXP_ID then do;
        CRM_VALUE_STD   = 0;
        CRM_VALUE_OTHER = 0;
    end;

    if CRM_ELIGIBLE_STD_FLG = 1 then 
        CRM_VALUE_STD   = sum(CRM_VALUE_STD, CRM_VALUE);
    else
        CRM_VALUE_OTHER = sum(CRM_VALUE_OTHER, CRM_VALUE);

    if last.EXP_ID then output;

    run;    

/*- 4: assign RW buckets according to ratio of adjustment to unsecured portion -*/
    data &RESLIB1..exp_in_def_std(drop=NETTING_SET_RK);    
    merge _exp_in_def_(in=m) 
          _exp_in_def_crm_total_(in=t);
   
    by EXP_ID;   
    
    if not t then do;
       CRM_VALUE_STD   = 0;
       CRM_VALUE_OTHER = 0;
    end;
    
    %if "&STD_PASTDUE_RATIO_TEST_DENOM" = "UNSECURED" %then %do;
        ADJUSTMENT_PCT = min(1000, sum(0, SPECIFIC_PROVISION) / max(0.001, sum(EAD, - CRM_VALUE_STD)) * 100); 
    %end;
    %else %do;
    if not (EAD in (.,0)) then
        ADJUSTMENT_PCT = min(1000, sum(0, SPECIFIC_PROVISION) / EAD * 100);
    %end;          

    
    STD_RW_BUCKET = put(ADJUSTMENT_PCT, MAP_STD_RW_BUCKET_PASTDUE.);
  

/*-5: get past due bucket to immovable properties linked to exposures in default. */    
    /*- get CRM_IDs linked to exposures that are in default -*/
    proc sql; 
    create table crm_in_def_std as 
        select
            ecl.CRM_ID, min(ADJUSTMENT_PCT) as MIN_ADJUSTMENT_PCT
        from &RESLIB1..exp_in_def_std as eids
             inner join RD_POS.exposure_crm_link as ecl
             on eids.EXP_ID = ecl.EXP_ID
        group by ecl.CRM_ID
        order by ecl.CRM_ID;
    quit;

    /*- update the CRM RW bucket when the CRM is immovable property -*/
    data &RESLIB1..all_crms(drop=rc MIN_ADJUSTMENT_PCT);
        length CRM_ID $32 MIN_ADJUSTMENT_PCT 8.;
        if _N_ = 1 then do;
            declare hash h(hashexp:6, dataset: "crm_in_def_std");
            rc = h.defineKey ("CRM_ID");
            rc = h.defineData("MIN_ADJUSTMENT_PCT");
            rc = h.defineDone();
            call missing(CRM_ID, MIN_ADJUSTMENT_PCT) ; 
        end;
        
        set &RESLIB1..all_crms;       
        if STD_PRODUCT_CLASS = 'IMM_PROPERTY' and CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' then do;
            rc = h.Find();
            if rc = 0 then do ;
               if STD_PRODUCT_SUBCLASS = 'RESIDENTIAL' then STD_RW_BUCKET_PASTDUE = put(MIN_ADJUSTMENT_PCT, MAP_STD_RW_BUCKET_PASTDUE_RME.); 
               else if STD_PRODUCT_SUBCLASS = 'COMMERCIAL' then STD_RW_BUCKET_PASTDUE = put(MIN_ADJUSTMENT_PCT, MAP_STD_RW_BUCKET_PASTDUE_CME.);
            end ;
        end;
         
    run;


/*- 6: update risk weight bucket for exposures in default  to &RESLIB1..all_exposures table. -*/
    data &RESLIB1..all_exposures;
        merge &RESLIB1..all_exposures(in=exp) &RESLIB1..exp_in_def_std(keep=EXP_ID STD_RW_BUCKET IN_DEFAULT_FLG in=def);
        by EXP_ID;
    run;

    %local N_DEFAULT_STD;
    %let N_DEFAULT_STD = %attrn(&RESLIB1..exp_in_def_std, NOBS);

    %if &N_DEFAULT_STD > 0 %then %do;
        %put ;
        %print_msg(note,2,361);
        %print_msg(note,2,362);
        %print_msg(note,2,363);
        %put ;
    %end;

    %EXIT :   

%mend r00_post_mtm_3_crd4; 
