/*****************************************************************************
 * Copyright (c) 2014 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: r00_post_mtm_6_crd4.sas
 *
 * PURPOSE: Check for granularity criteria for regulatory retail portfolio. 
 *          Check for SME capital deduction qualifying criteria.
 *   USAGE: 
 *
 *   NOTES: 5/23/2014
 *          Updated retail granularity criteria based on CRR regulations.
 *          Added SME capital deduction qualifying criteria.
 *          More qualifying criteria can be added here as needed.
 *          
 *
 *****************************************************************************/

%macro r00_post_mtm_6_crd4;

    %if APPROACH = 'BASEL_I' %then %goto EXIT;

/*- qualifying criteria for regulatory retail portfolio -*/
    %local TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB;
    %let TOTAL_RETAIL_EXPOSURE_STD  = 0;
    %let TOTAL_RETAIL_EXPOSURE_IRB  = 0;

/*- 1: get all non-sec-underlying retail exposures -*/
    data _all_retail_(keep=EXP_ID EAD COUNTERPARTY_RK COUNTERPARTY_GROUP APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                           EXT_ISSUE_MAP_RATING SHORT_TERM_ASSESS_FLG STD_RW_BUCKET EXT_MAP_RATING
                           STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS
                           PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG CPTY_GROUP_ID
                      );
    length CPTY_GROUP_ID $32. ;                 
    set &RESLIB1..all_exposures end=last;
    where COUNTERPARTY_CLASS = 'RETAIL'  and SEC_UNDERLYING_EXP_FLG NE 1
          and ((APPROACH='STD' and STD_PRODUCT_CLASS not in ('SEC', 'SECURITIZATION')) 
          or (APPROACH='IRB' and IRB_PRODUCT_CLASS not in ('SEC', 'SECURITIZATION')));
    
    /*- retail granularity check is based on obligor client or group of connected clients: CRR Article 123(c) -*/
    if compress(COUNTERPARTY_GROUP) ne '' then CPTY_GROUP_ID = COUNTERPARTY_GROUP;
    else CPTY_GROUP_ID= trim(left(put(COUNTERPARTY_RK, best.))) ; 
    
    retain TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB 0;
    
    /*- total exposure for STD regulatory portfolio -*/    
        if APPROACH = 'STD' and not (STD_PRODUCT_CLASS in ('MORTGAGE', 'LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS ='RESIDENTIAL')
        then
            TOTAL_RETAIL_EXPOSURE_STD = sum(TOTAL_RETAIL_EXPOSURE_STD, max(EAD,0));
    /*- total exposure for IRB regulatory portfolio -*/
        else if APPROACH = 'IRB' then
            TOTAL_RETAIL_EXPOSURE_IRB = sum(TOTAL_RETAIL_EXPOSURE_IRB, max(EAD,0));

    if last then do;
        /*- create macro variable for total exposure for regulatory retail portfolio -*/
        /*- under STANDARDIZED approach: Basel II - 70 -*/    
        call symput('TOTAL_RETAIL_EXPOSURE_STD', TOTAL_RETAIL_EXPOSURE_STD);
        /*- create macro variable for total exposure for regulatory retail portfolio -*/
        /*- under IRB approach: Basel II - 231 - 234 -*/        
        call symput('TOTAL_RETAIL_EXPOSURE_IRB', TOTAL_RETAIL_EXPOSURE_IRB);
    end;
    run;

/*- 2: get retail portfolio size under STD and IRB approaches for each CPTY_GROUP_ID -*/
/*-    count number of defaulted exposures for each COUNTERPARTY_RK                  -*/         
    proc sort data=_all_retail_;
        by CPTY_GROUP_ID COUNTERPARTY_RK;
    run ;
    
    data cpty_retail(keep=CPTY_GROUP_ID TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_STD TOTAL_RETAIL_REVOLV_CPTY_IRB) 
         cpty_rk_retail_std(keep=CPTY_GROUP_ID COUNTERPARTY_RK NUM_DEFAULTED_EXP_CPTY_STD)
         cpty_rk_retail_irb(keep=CPTY_GROUP_ID COUNTERPARTY_RK NUM_DEFAULTED_EXP_CPTY_IRB);
    set _all_retail_(keep=EAD CPTY_GROUP_ID COUNTERPARTY_RK APPROACH STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS
              PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG IRB_PRODUCT_SUBCLASS);
    by CPTY_GROUP_ID COUNTERPARTY_RK;
    
    retain TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_STD TOTAL_RETAIL_REVOLV_CPTY_IRB 
           NUM_DEFAULTED_EXP_CPTY_STD NUM_DEFAULTED_EXP_CPTY_IRB 0;

    if first.CPTY_GROUP_ID then do;
        TOTAL_RETAIL_CPTY_STD = 0;
        TOTAL_RETAIL_CPTY_IRB = 0;
        TOTAL_RETAIL_REVOLV_CPTY_STD = 0;
        TOTAL_RETAIL_REVOLV_CPTY_IRB = 0;
        if first.COUNTERPARTY_RK then do ;
           NUM_DEFAULTED_EXP_CPTY_STD = 0 ; 
           NUM_DEFAULTED_EXP_CPTY_IRB = 0 ;
        end ;   
    end;

    if APPROACH = 'STD' then do;
        /*- total retail exposure under STD for every CPTY_GROUP_ID  -*/
        if not (STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS ='RESIDENTIAL') 
           then TOTAL_RETAIL_CPTY_STD = sum(TOTAL_RETAIL_CPTY_STD, EAD);
        /*- total retail revolving exposure under STD for every CPTY_GROUP_ID  -*/ 
        if STD_PRODUCT_CLASS = 'REVOLVING' then
            TOTAL_RETAIL_REVOLV_CPTY_STD = sum(TOTAL_RETAIL_REVOLV_CPTY_STD, EAD);
       
        if IN_DEFAULT_FLG = 1 or PAST_DUE_CLAIM_FLG = 1 then NUM_DEFAULTED_EXP_CPTY_STD +1 ; 
        
        if last.COUNTERPARTY_RK then do ;    
               output cpty_rk_retail_std ;
               NUM_DEFAULTED_EXP_CPTY_STD = 0;
        end ;
    end;
        
    else do;
        /*- total retail exposure under IRB for every CPTY_GROUP_ID  -*/
        /* residential real estate are excluded from the total: B2 231 and CRD art.86-4(a) */
        if not (IRB_PRODUCT_CLASS = 'MORTGAGE' and IRB_PRODUCT_SUBCLASS = 'RESIDENTIAL') then
        TOTAL_RETAIL_CPTY_IRB = sum(TOTAL_RETAIL_CPTY_IRB, EAD);
        /*- total retail revolving exposure under IRB for every CPTY_GROUP_ID  -*/
        if IRB_PRODUCT_CLASS = 'REVOLVING' then
            TOTAL_RETAIL_REVOLV_CPTY_IRB = sum(TOTAL_RETAIL_REVOLV_CPTY_IRB, EAD);
        
        if IN_DEFAULT_FLG = 1 or PAST_DUE_CLAIM_FLG = 1 then NUM_DEFAULTED_EXP_CPTY_IRB +1 ;
        
        if last.COUNTERPARTY_RK then do ;    
              output cpty_rk_retail_irb ;
              NUM_DEFAULTED_EXP_CPTY_IRB = 0;
        end ;
    end;

    

    if last.CPTY_GROUP_ID then output cpty_retail;
    run;
    
/*- 3: retail portfolio qualifying criteria check -*/    
    /*- update to get count of defaulted exposures per counterparty -*/
    data _all_retail_ ;
       merge _all_retail_  
             cpty_rk_retail_std
             cpty_rk_retail_irb ;
       by CPTY_GROUP_ID COUNTERPARTY_RK ;
    run ;   

    data &RESLIB1..retail_granularity_exception
        (where=(CPTY_CLASS_CHANGED_FLG = 1 or IRB_PRODUCT_CLASS_CHANGED_FLG = 1)
         keep=EXP_ID CPTY_GROUP_ID COUNTERPARTY_RK APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS EXT_ISSUE_MAP_RATING STD_RW_BUCKET 
              IRB_PRODUCT_CLASS CPTY_CLASS_CHANGED_FLG IRB_PRODUCT_CLASS_CHANGED_FLG EXT_MAP_RATING
              NUM_DEFAULTED_EXP_CPTY_STD NUM_DEFAULTED_EXP_CPTY_IRB IN_DEFAULT_FLG 
         index=(EXP_ID)
        );    
    merge _all_retail_ cpty_retail  ;
    by CPTY_GROUP_ID;
    
    CPTY_CLASS_CHANGED_FLG = 0;
    IRB_PRODUCT_CLASS_CHANGED_FLG = 0;
 
/*- qualifying criteria for regulatory retail portfolio under STD approach -*/
%if "&USE_UNDRAWN_RETAIL_GRAN_CHECK" = "YES" %then %do ;
    if FACILITY_FLG ne 1 ;
%end ;

    if APPROACH = 'STD' then do;
        if not (STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS = 'RESIDENTIAL') then do;
            if /*(TOTAL_RETAIL_CPTY_STD > ((&STD_RETAIL_CPTY_REL_LIMIT / 100) *  (&TOTAL_RETAIL_EXPOSURE_STD + 0))) or*/ 
               (TOTAL_RETAIL_CPTY_STD > &STD_RETAIL_CPTY_ABS_LIMIT) then do;

                COUNTERPARTY_CLASS = 'CORPORATE';
                if COUNTERPARTY_SUBCLASS = 'RETAIL' then COUNTERPARTY_SUBCLASS = 'CORPORATE';
                CPTY_CLASS_CHANGED_FLG = 1;                

                /*- update for in default status: for corporate exposures, if one exposure is defaulted,   -*/
                /*- then all other exp of the counterparty are considered defaulted: CRR Article 178       -*/
                if NUM_DEFAULTED_EXP_CPTY_STD >0 then IN_DEFAULT_FLG = 1 ; 
                
                /*- only non past dues are assigned a new rw_bucket as bucket for past dues assigned below -*/
                if not (PAST_DUE_CLAIM_FLG = 1 or IN_DEFAULT_FLG = 1)
                then do; 
                        if SHORT_TERM_ASSESS_FLG = 1 then
                           STD_RW_BUCKET = put(EXT_ISSUE_MAP_RATING, $MAP_STD_RW_BUCKET_SHORT.);
                        else
                           STD_RW_BUCKET = put(EXT_ISSUE_MAP_RATING, $MAP_STD_RW_BUCKET_CORPORATE.);
                end;
            end;
        end;    
    end;
     
   else if APPROACH ne 'STD' then do; 
/*- qualifying criteria for regulatory retail portfolio under IRB approach -*/        
        if COUNTERPARTY_SUBCLASS = 'SME' then do;
            if TOTAL_RETAIL_CPTY_IRB >= &IRB_RETAIL_SME_CPTY_ABS_LIMIT then do;
                COUNTERPARTY_CLASS = 'CORPORATE';
                CPTY_CLASS_CHANGED_FLG = 1;
                /*- change approach if COUNTERPARTY_CLASS is changed to CORPORATE -*/
                APPROACH = 'IRBA';
            end;
        end;
/*- qualifying criteria for regulatory retail revolving portfolio under IRB approach -*/        
        else if IRB_PRODUCT_CLASS = 'REVOLVING' then do;
            if TOTAL_RETAIL_REVOLV_CPTY_IRB > &IRB_RETAIL_QRRE_CPTY_ABS_LIMIT then do;
                IRB_PRODUCT_CLASS = 'OTHER';        
                IRB_PRODUCT_CLASS_CHANGED_FLG = 1;
            end;
        end;
    end;

run;

    proc sort data=rd_pos.counterparty out=&RESLIB1..counterparty;
    by CPTY_RK;
    run;

/*- 4: update for these retail exposures -*/

/*- sort the data set before the merge if netted repo-style or -*/
/*- off-balance sheet exposures have been added -*/

  %if &N_REPO_NET > 0 or &N_OBSI_NET > 0 %then %do ;
        proc sort data = &RESLIB1..all_exposures ;
            by EXP_ID ;
    run ;
    %end 

    %local NCHANGED;
    %let NCHANGED = %attrn(&RESLIB1..retail_granularity_exception, NOBS);
    %if &NCHANGED > 0 %then %do;
    
        data &RESLIB1..all_exposures;
        merge &RESLIB1..all_exposures(in=exp)
              &RESLIB1..retail_granularity_exception(keep=EXP_ID CPTY_GROUP_ID COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                                                          IRB_PRODUCT_CLASS STD_RW_BUCKET APPROACH
                                                          IN_DEFAULT_FLG CPTY_CLASS_CHANGED_FLG NUM_DEFAULTED_EXP_CPTY_STD in=retail)
              ;
        by EXP_ID;
        run;
     
       /* update rw bucket for all exposures to a defaulted corporate counterparty */
        %r00_post_mtm_3_crd4 ;
     
     
       /* update counterparty rw bucket */      
        proc sort data=&RESLIB1..retail_granularity_exception (where=(CPTY_CLASS_CHANGED_FLG=1))
                  out=cpty_retail_1 (keep=COUNTERPARTY_RK EXT_MAP_RATING COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS) nodupkey;
        by COUNTERPARTY_RK;
        run;
        
        data &RESLIB1..counterparty(drop=EXT_MAP_RATING);
        merge &RESLIB1..counterparty (in=a)
              cpty_retail_1(in=b rename=(COUNTERPARTY_RK=CPTY_RK COUNTERPARTY_CLASS=CPTY_CLASS
                                         COUNTERPARTY_SUBCLASS=CPTY_SUBCLASS));
        by CPTY_RK;
        if a;
        
        if b then 
           STD_RW_BUCKET_CPTY = put(EXT_MAP_RATING, $MAP_STD_RW_BUCKET_CORPORATE.);
        run;
     
  /*- 5: reporting -*/
          %put ;
          %print_msg(warning,2,50);
          %print_msg(warning,2,51);
          %print_msg(warning,2,52);
          %put ;
     
    %end;

/*************************************************************************/
/*- Capital deduction qualifying criteria for regulatory SME portfolio  -*/
/***********************************************************************-*/
   /*-  get all non-sec-underlying sme exposures -*/
    data  _all_sme_(keep = EXP_ID EAD COUNTERPARTY_RK COUNTERPARTY_GROUP COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                           STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS 
                           SME_SUPPORTING_FACTOR_FLG CPTY_GROUP_ID
                    index=(CPTY_GROUP_ID)
                   );
       length CPTY_GROUP_ID $32. ;                  
       set &RESLIB1..all_exposures ;
       where COUNTERPARTY_CLASS in ('RETAIL', 'CORPORATE') and COUNTERPARTY_SUBCLASS in ('SME') and SEC_UNDERLYING_EXP_FLG ne 1 ;
              
       /*- SME capital deduction qualification check is based on obligor client or group of connected clients: CRR Article 501(c) -*/
       if compress(COUNTERPARTY_GROUP) ne '' then CPTY_GROUP_ID = COUNTERPARTY_GROUP;
       else CPTY_GROUP_ID= trim(left(put(COUNTERPARTY_RK, best.))) ;    
    run ;
    
    data _sme_cpty_ ;
       set  _all_sme_ ;
       by CPTY_GROUP_ID ;
       
       retain TOTAL_SME_CPTY 0 ;

       if first.CPTY_GROUP_ID then do;
           TOTAL_SME_CPTY = 0;
       end;
    
       /*- total SME exposure for all qualifying SMEs in a CPTY_GROUP_ID: -*/
       /*- claims secured by residential property are excluded from the total: CRR Article 501(c) -*/
       if not ((STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS ='RESIDENTIAL') or
               (IRB_PRODUCT_CLASS = 'MORTGAGE' and IRB_PRODUCT_SUBCLASS = 'RESIDENTIAL'))
        
        then  TOTAL_SME_CPTY = sum(TOTAL_SME_CPTY, EAD);
        

       if last.CPTY_GROUP_ID then output ;
    run ;
    
    data  &RESLIB1..all_sme ;
       merge _all_sme_   _sme_cpty_(keep = CPTY_GROUP_ID TOTAL_SME_CPTY) ;
       by CPTY_GROUP_ID ;
   
      /*- SME portfolio capital deduction qualifying criteria check -*/ 
        if SME_SUPPORTING_FACTOR_FLG = 1 and TOTAL_SME_CPTY <= &SME_CPTY_LIMIT then SME_CAP_DEDUCTION_TRT_FLG = 1 ;
        else SME_CAP_DEDUCTION_TRT_FLG = 0 ;
    run;
    
    
    %local NSME;
        %let NSME = %attrn(&RESLIB1..all_sme, NOBS);
    %if &NSME > 0 %then %do;
    
       proc sort data= &RESLIB1..all_sme ;
              by EXP_ID ;
       run ;

       data &RESLIB1..all_exposures;
           merge &RESLIB1..all_exposures(in=exp)
                 &RESLIB1..all_sme(keep=EXP_ID SME_CAP_DEDUCTION_TRT_FLG in=sme)
                 ;
           by EXP_ID;
       run;
 
    %end ;


    %EXIT :   

%mend r00_post_mtm_6_crd4;
