/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: r00_post_mtm_7_crd4.sas
 *
 * PURPOSE: Assign instrument types for regulatory calculations.
 *          
 *
 *   USAGE: 
 *
 *   NOTES: May 2013:  Modified INSTTYPE for STD exposures for CRD-IV.
 *          Oct. 2013: Reorder the STD exposure classes assignment. 
 *
 *****************************************************************************/


%macro r00_post_mtm_7_crd4;

/*- 1: create regulatory instrument types for exposures -*/
    data &RESLIB1..all_exposures
       &RESLIB1..other_items_unclassified;  
 
    length INSTTYPE $32.;
    length IRB_SCALING_FACTOR 8.;
    
    set &RESLIB1..all_exposures;

    /*- Default case -*/
    INSTTYPE = compress('EXP_' || COUNTERPARTY_CLASS || '_' || APPROACH);
    
    /*- Create an effective maturity variable for CVA standardized calculations. This has -*/
    /*- to be done since the effective maturity can be set to missing if the -*/
    /*- exposures credit risk charge is calculated under standardized -*/
    CVA_STD_MAT_EFF = MAT_EFF ;  

    /*- Special cases -*/
    if APPROACH = 'BASEL_I' then INSTTYPE = 'EXP_BASEL_I';

    /* Counterparty credit risks: CCP related exposures, CRR article 305 -*/
    else if "&TRADE_EXP_CCP_PREF_RW_TRTMNT" = "YES" and TRADE_EXP_FLG = 1 and CCP_CPTY_SUBCLASS = 'QCCP'
            and BANKRUPTCY_REMOTE_FLG = '1' and ASSET_TRANSFERRABLE_FLG = 1
       then INSTTYPE = 'EXP_CCR_CCP';
   
    
    else if APPROACH ='STD_IAA' then   
            INSTTYPE = 'EXP_SECURITIZATION_IRB'; 
                
    /* STD exposure class assignments are based on 3.2.3 of Annex II-Instructions_Own funds.docx  */
    else if APPROACH = 'STD' then do;
        MAT_EFF = .; 
        
        /*- Securitization exposures -*/
        if STD_PRODUCT_CLASS in ('SEC', 'SECURITIZATION') then 
            INSTTYPE = 'EXP_SECURITIZATION_STD';
        
        /*- Items associated with particular high risk -*/
        else if HIGH_RISK_CPTY_FLG = 1 or STD_PRODUCT_CLASS = 'HIGH_RISK' then INSTTYPE = 'EXP_HIGH_RISK_STD';
        
        /*- Equity exposures -*/  
        else if STD_PRODUCT_CLASS = 'EQUITY' then INSTTYPE = 'EXP_EQUITY_STD' ;
        
        /*- Exposures in default -*/
        else if PAST_DUE_CLAIM_FLG = 1 or IN_DEFAULT_FLG =1 then INSTTYPE = 'EXP_DEFAULTED_STD';
        
        /*- CIUs -*/
        else if STD_PRODUCT_CLASS eq 'CIU' then INSTTYPE = 'EXP_CIU_STD';
        
        /*- Covered Bonds -*/
        else if STD_PRODUCT_CLASS eq 'COVERED_BOND' then INSTTYPE = 'EXP_COV_BOND_STD';
                
        /*- Exposures secured by immovable properties -*/
        else if STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') then do;
            if STD_PRODUCT_SUBCLASS = 'RESIDENTIAL' then INSTTYPE = 'EXP_RME_STD';
            else if STD_PRODUCT_SUBCLASS = 'COMMERCIAL' then INSTTYPE = 'EXP_CME_STD';
        end;
        
         /*- Lease residual value   -*/
        else if STD_PRODUCT_SUBCLASS = 'LEASE_RESIDUAL' then INSTTYPE = 'EXP_LEASE_RESIDUAL_STD';
        
        /*- Other assets -*/
        else if STD_PRODUCT_CLASS = 'OTHER_ASSETS' then INSTTYPE = 'EXP_OTHER_ASSETS_STD';
                     
        
        /*-  Sovereign exposures are based on counterparty_subclasses -*/ 
        else if COUNTERPARTY_CLASS eq 'SOVEREIGN' then do ;
             INSTTYPE = compress('EXP_' || COUNTERPARTY_SUBCLASS || '_' || APPROACH) ;
             if COUNTERPARTY_SUBCLASS in ('CENTRAL_GOV','CENTRAL_BANK') then INSTTYPE = compress('EXP_' || 'CENTGOV' || '_' || APPROACH) ;
             else if COUNTERPARTY_SUBCLASS in ('REGIONAL_GOV','LOCAL_GOV','RELIGIOUS_ORG') then INSTTYPE = compress('EXP_' || 'REGGOV' || '_' || APPROACH) ;
             else if COUNTERPARTY_SUBCLASS in ('MDB','HR_MDB') then INSTTYPE = compress('EXP_' || 'MDB' || '_' || APPROACH) ;   
        end ; 
        
        else if COUNTERPARTY_CLASS = ' ' then do;
            INSTTYPE = 'EXP_OTHER_ASSETS_STD';                       
            output &RESLIB1..other_items_unclassified;
        end;
          
    end;        
    
    /*- IRB exposures -*/
    /*- retail exposures -*/
    else if APPROACH = 'IRB' then do;
        if RECEIVABLE_EXPOSURE_FLG = 1 then do;
            INSTTYPE = 'EXP_RETAIL_REC_IRB';
        end;
        else do;
            INSTTYPE = 'EXP_RETAIL_IRB';
        end;
    end;
    
    /*- specialized lending (HVCRE) and purchased receivables -*/
    else if APPROACH in ('IRBF', 'IRBA') then do;  
        if APPROACH = 'IRBF' and "&IRBF_M_SUPERVISORY" = "YES" then do;
           MAT_EFF = &IRBF_M;
           if IRB_PRODUCT_CLASS = 'REPO_STYLE' then
              MAT_EFF = &IRBF_M_REPO;
        end;
        
        if IRB_PRODUCT_CLASS = 'SL' and IRB_PRODUCT_SUBCLASS = 'HVCRE' then 
            INSTTYPE = compress('EXP_CORPORATE_HVCRE_' || APPROACH);
        else if IRB_PRODUCT_CLASS = 'PURCHASED_REC' then do;
            INSTTYPE = compress('EXP_CORPORATE_REC_' || APPROACH);           
            if APPROACH = 'IRBF' then MAT_EFF = &IRBF_M_REC_DEFAULT;
        end;
        else if IRB_PRODUCT_CLASS = 'OTHER_ASSETS' and IRB_PRODUCT_SUBCLASS in ('OTHER', 'OTHER_NON_CREDIT_OBLG') then
               INSTTYPE = 'EXP_IRB_OTHER_ASSETS';
        else if IRB_PRODUCT_CLASS = 'OTHER_ASSETS' and IRB_PRODUCT_SUBCLASS in ('CASH', 'GOLD', 'CASH_GOLD_BULLION_EXP') then
               INSTTYPE = 'EXP_IRB_CASH_GOLD';           
    end;
    
    /*- securitization exposures -*/
    else if APPROACH in ('IRB_RBA', 'IRB_IAA') then do;
        MAT_EFF = .;
        INSTTYPE = 'EXP_SECURITIZATION_IRB';             
    end; 
    
    /*- slotting criteria for specilized lending -*/
    else if APPROACH = 'IRB_SLOTTING' then do;
        MAT_EFF = &IRBF_M;
        if IRB_PRODUCT_SUBCLASS = 'HVCRE' then INSTTYPE = 'EXP_HVCRE_IRB_SLOTTING';
        else INSTTYPE = 'EXP_SL_IRB_SLOTTING';
    end;
    
    /*- equity exposures under IRB approach -*/
    else if APPROACH in ('IRB_SIMPLE_RW', 'IRB_INT_MODELS', 'IRB_PD_LGD') then do;
        MAT_EFF = &IRB_M_EQUITY;
        INSTTYPE = compress('EXP_EQ_' || APPROACH);
    end;
    
    /*- define scaling factor according to approach -*/
    IRB_SCALING_FACTOR = input(put(APPROACH, $MAP_CAPITAL_SCALING_FACTOR.), best12.);
    
    /*- define default case scaling factor according to approach -*/
    IRB_DEFAULT_SCALING_FACTOR = input(put(APPROACH, $MAP_CAPITAL_DEF_SCALING_FACTOR.), best12.);

    /*- a special case is the securitization pool capital charge for the originator bank -*/
    if POSITION_CATEGORY = 'SEC_POOL' then do;
        INSTTYPE = 'EXP_SECURITIZATION_POOL';
        IRB_SCALING_FACTOR = 1; /* scaling factor is one no matter what approach */  
    end;
    
    /*- lease residual value -*/
    if IRB_PRODUCT_SUBCLASS = 'LEASE_RESIDUAL' then do;
        INSTTYPE = 'EXP_LEASE_RESIDUAL_VALUE';
        IRB_SCALING_FACTOR = 1; /* scaling factor is one no matter what approach */  
    end;

  output &RESLIB1..all_exposures ;

    run;

  %local NOTHERS;
  %let NOTHERS = %attrn(&RESLIB1..other_items_unclassified, NOBS);
  %if &NOTHERS > 0 %then %do;
  %put; 
    %print_msg(warning, 2, 305);
  %print_msg(warning, 2, 306);
  %print_msg(warning, 2, 307);
  %put;
  %end;   
  
   /* Invalidate exp data with missing instrument types */ 
      data  &RESLIB1..all_exposures  
          &RESLIB1..Invalid_exp_insttypes ; 
      set &RESLIB1..all_exposures  ; 
     if missing (INSTTYPE) or (INDEX(INSTTYPE, '#') NE 0) then  output &RESLIB1..Invalid_exp_insttypes ; 
     else output 
          &RESLIB1..all_exposures ; 
      run ; 
  

/*- 2: create regulatory instrument types for crms -*/    
    data &RESLIB1..all_crms;

    length INSTTYPE $32.;
    
    set &RESLIB1..all_crms;    
      
    select (CRM_TYPE);
    
        when ('FINANCIAL_COLLATERAL')
            INSTTYPE = 'CRM_FINANCIAL';

        when ('IRB_COLLATERAL') 
            INSTTYPE = 'CRM_IRB';
            
        when ('STD_PHYSICAL_COLLATERAL') 
            INSTTYPE = 'CRM_PHYSICAL_STD';
            
        when ('GUARANTEE')
            INSTTYPE = 'CRM_GUARANTEE';

        when ('CREDIT_DERIVATIVE')
            INSTTYPE = 'CRM_DERIVATIVE';

        when ('INSURANCE')
            INSTTYPE = 'CRM_INSURANCE';

        when ('NETTING') 
            INSTTYPE = 'CRM_NETTING';           
   
        otherwise;
        
    end;
    
    run;
 
 /* Invalidate crm data with missing instrument types */ 
      data  &RESLIB1..all_crms  
          &RESLIB1..Invalid_crms_types ; 
      set &RESLIB1..all_crms  ; 
     if missing (INSTTYPE) then  output &RESLIB1..Invalid_crms_types ; 
      else output 
          &RESLIB1..all_crms ; 
      run ; 

/*- 3: report -*/    
    %put ;
    %print_msg(note,2,53);

    %local NO_EXP;
    %let NO_EXP = %attrn(&RESLIB1..all_exposures, NOBS);

    proc summary data=&RESLIB1..all_crms missing;
    class CRM_TYPE;
    output out=crm_summary ;
    run;

    %print_msg(note,2,54);
    %print_msg(note,2,55);

    %put ;
    %print_msg(note,2,56);
    
    data _null_;
    set crm_summary;
    if _type_ = 1 then do ;
        call execute('%put &NOTE: ***' !! trim(left(_freq_)) !! ' ' !! CRM_TYPE !! ' ***;');
    end;
    run;

    %put ;


%mend r00_post_mtm_7_crd4;
