/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: r00_post_regulatory_6_crd4.sas
 *
 * PURPOSE: Handles exceptions to unrated securitization exposures 
 *          subject to standardized approach (CRR 253 - 256), e.g.
 *          look-through treatment.  Also adjusts the RW for securitization
            positions in the case of infringement (CRR 407).
 *
 *   USAGE: 
 *
 *   NOTES: 
 *          
 *
 *****************************************************************************/

%macro r00_post_regulatory_6_crd4;

/*- for those eap pool and unrated securitization exposures under STD -*/
    data _all_need_underlying_std;
    set &RESLIB1..all_exp_ur_sec_std all_exp_pool_std;
    run;
    
    data _all_pool_ids_need_und_std(keep=SECURITIZATION_POOL_RK);
        set _all_need_underlying_std;
    run;
    
    proc sort data=_all_need_underlying_std; by SECURITIZATION_POOL_RK tranche_loss_rank; run;
    proc sort data=_all_pool_ids_need_und_std nodup; by SECURITIZATION_POOL_RK ; run;

/*- exposures underlying unrated securitization exposures under STD -*/
    data  &RESLIB1..all_exp_sec_underlying_std;
    merge &RESLIB1..all_exp_sec_underlying(in=underlying) _all_pool_ids_need_und_std (in=ur_std keep=SECURITIZATION_POOL_RK);
    by SECURITIZATION_POOL_RK;
    if underlying and ur_std;
    /*- For underlying IRB exposures, need to add EL RW for consistency with STD -*/
    if APPROACH =: 'IRB' then RW_EXP = sum(RW_EXP, EL / &CAPITAL_RATIO);
    run;
       
/*- if neither underlying nor unrated available, update unnecessary -*/
    %local N_SEC_UNDERLYING_STD N_UR_SEC_STD N_UR_SEC_STD_RW_UPDATE;
    %let N_SEC_UNDERLYING_STD = %attrn(&RESLIB1..all_exp_sec_underlying_std, NOBS);
    %let N_UR_SEC_STD         = %attrn(_all_need_underlying_std, NOBS);    
    %let N_UR_SEC_STD_RW_UPDATE = 0;
    
    %if &N_SEC_UNDERLYING_STD = 0 or &N_UR_SEC_STD = 0 %then %goto NO_UPDATE_UR_SEC;
    
/*- 2: find min, max, and weighted-average RW for securitization pools under STD -*/
    proc summary data = &RESLIB1..all_exp_sec_underlying_std;
    by SECURITIZATION_POOL_RK;
    weight EAD;
    var RW_EXP;
    by SECURITIZATION_POOL_RK;
    output out  = &RESLIB1..rw_sec_underlying_ur_std(keep=SECURITIZATION_POOL_RK MIN_RW WA_RW MAX_RW)
           min  = MIN_RW
           mean = WA_RW
           max  = MAX_RW
           ;
    run;

/*- compute concentration ratio -*/
/*- sum total amount in each pool -*/
    /* get rated tranches */
    data _tranches_conc_ratio_std;
    	set &RESLIB1..all_exposures ;
    	where SECURITIZATION_EXPOSURE_FLG = 1 and APPROACH ='STD'  
    	     and STD_PRODUCT_SUBCLASS not in ('EAP_ORIGINATOR_INT','EAP_INVESTORS_INT', 'DERIVATIVE') 
    	     and STD_PRODUCT_SUBCLASS not in ('ELQD_FACILITY','FACILITY','ECA_FACILITY') ;
    run;
    
    /* append together rated tranches and unrated tranches */
	proc append data=_all_need_underlying_std base = _tranches_conc_ratio_std;
	run;

    proc sort data = _tranches_conc_ratio_std out = _uniq_conc_ratio_std nodupkey ;
        by  securitization_pool_rk tranche_loss_rank ;
    run ;
    
    proc summary data = _uniq_conc_ratio_std;
        var tranche_face_value_amt;
        by securitization_pool_rk;
        output out= all_tranches sum=all_tranches_sum;
    run;

    /*- merge pool totals into dataset of all tranches where at least one tranche in the pool is unrated -*/
    data tranches_with_totals;
        merge _uniq_conc_ratio_std all_tranches (keep = securitization_pool_rk all_tranches_sum);
        by securitization_pool_rk ;
        if tranche_loss_rank ne .;
    run;

    /*- compute concentration ratio at the tranche level -*/
    data concentration_ratio(keep = securitization_pool_rk conc_ratio tranche_loss_rank);
        set tranches_with_totals;
        by securitization_pool_rk;
        retain sum_junior_tranches;
        if first.securitization_pool_rk then sum_junior_tranches = 0;
        sum_junior_tranches = sum(sum_junior_tranches , tranche_face_value_amt);
        if sum_junior_tranches ne 0 then
            conc_ratio = all_tranches_sum/sum_junior_tranches;
    run;

    /*- merge calculated concentration ratio back into the dataset of tranches that need underlying (unrated tranches) -*/    
    data _all_need_underlying_std;
    merge _all_need_underlying_std (in=a) concentration_ratio;
    by securitization_pool_rk tranche_loss_rank;
	if a;
    run;     
    

/*- 3: update RW for ur securitization exposures under STD in Basel II - 571 - 576 -*/
/*-    update only when there are no user-supplied values                          -*/
    data _all_need_underlying_std(drop = MIN_RW WA_RW MAX_RW index=(EXP_ID));
    merge _all_need_underlying_std &RESLIB1..rw_sec_underlying_ur_std;
    by SECURITIZATION_POOL_RK;    
    
    if AVG_RISK_WEIGHT = . then AVG_RISK_WEIGHT = WA_RW;
    if MIN_RISK_WEIGHT = . then MIN_RISK_WEIGHT = MIN_RW;
    if MAX_RISK_WEIGHT = . then MAX_RISK_WEIGHT = MAX_RW;
    run;
    
%NO_UPDATE_UR_SEC :    
/*- separate data -*/
    data _pool_ ur_sec;
    set _all_need_underlying_std;
    if POSITION_CATEGORY = 'SEC_POOL' or not missing(COVER_SECURITIZATION_POOL_RK)  then
       output _pool_;
    else 
       output ur_sec;
    run;
    
/*- assign risk weights -*/
    data all_exp_pool_std;
    set _pool_;
/*- pool with eap -*/
    if POSITION_CATEGORY = 'SEC_POOL' then do ;
       RW_EXP = AVG_RISK_WEIGHT;
       if RW_EXP ne . then RW_LOOK_THROUGH_UPDATE_FLG = 1; 
       else if RW_EXP = . then RW_EXP = &RW_UNRATED_SECURITIZATION;
    end ; 
    
/*- liquidity facility  -*/
    else do;
       if STD_PRODUCT_CLASS ne 'DERIVATIVE' then do ;
          RW_EXP = MAX_RISK_WEIGHT;
          if RW_EXP ne . then RW_LOOK_THROUGH_UPDATE_FLG = 1; 
          else if RW_EXP = . then RW_EXP = &RW_UNRATED_SECURITIZATION; 
       end ;   
    end;
    run;
    
    proc append base=&RESLIB1..all_exposures data=all_exp_pool_std force;run;
    
/*- check if bank retains any first loss tranche in ABCP program exposure -*/
    data ur_sec;
    merge ur_sec (in=a) first_loss_abcp (in=b);
    by SECURITIZATION_POOL_RK;
    if a;
    if b then FIRST_LOSS_RETAINED=1;
    else FIRST_LOSS_RETAINED=0;
    run;
    
/*- other unrated securitization exposures -*/
    data &RESLIB1..all_exp_ur_sec_std;
    RW_UPDATED_FLG = 1;
    RW_LOOK_THROUGH_UPDATE_FLG = 0;
    set ur_sec;   

    /*- CRR 254: 2nd loss ABCP -*/ 
    if SEC_STRUCTURE_SUBTYPE = 'ABCP' and TRANCHE_LOSS_RANK >= 2 and MAX_RISK_WEIGHT NE . 
            and FIRST_LOSS_RETAINED = 0 then do;
        RW_EXP = max(100, MAX_RISK_WEIGHT);
        RW_LOOK_THROUGH_UPDATE_FLG = 1;
    end;

    /*- CRR 255: ur eligible liquidity facility -*/
    else if STD_PRODUCT_SUBCLASS = 'ELQD_FACILITY' and MAX_RISK_WEIGHT NE . then do ;
        RW_EXP = MAX_RISK_WEIGHT;
        RW_LOOK_THROUGH_UPDATE_FLG = 1;
    end ;    

    /*- CRR 253: ur use average risk weight of underlyings if available-*/
    else if AVG_RISK_WEIGHT NE . then do;
        RW_EXP = AVG_RISK_WEIGHT * conc_ratio;
        RW_LOOK_THROUGH_UPDATE_FLG = 1;
    end;
    
    /*- All other unrated securitization exposures:  -*/
    /*- Article 251 of CRR: Subject to Articles 252 to 255, an unrated securitisation position  -*/
    /*  shall be calculated by applying a risk weight of 1250 %.                                -*/
    else 
          RW_EXP = &RW_UNRATED_SECURITIZATION ; 
    run;

    proc sql noprint;
    select count(*) into :N_UR_SEC_STD_RW_UPDATE
    from &RESLIB1..all_exp_ur_sec_std(keep=RW_UPDATED_FLG)
    where RW_UPDATED_FLG = 1;
    quit;

    %if &N_UR_SEC_STD_RW_UPDATE > 0 %then %do;
        %put ;
        %print_msg(note,2,67);
        %print_msg(note,2,68);
        %print_msg(note,2,69);
        %put ;
    %end;
   
/*- 4: get these updated securitization exposure back    -*/
    proc append base=&RESLIB1..all_exposures 
                data=&RESLIB1..all_exp_ur_sec_std(drop=RW_UPDATED_FLG FIRST_LOSS_RETAINED) force;
    run;

/*- update exposure-crm links only when necessary -*/
    /*- if no update done to unrated, no need to update -*/
    %if &N_UR_SEC_STD_RW_UPDATE = 0 %then %do;
        %goto EXIT;
    %end;
    
    %if &N_UR_SEC_STD_RW_UPDATE > 0 %then %do;
        proc sort data=&RESLIB1..all_exp_ur_sec_std;
        by EXP_ID;
        run;

        proc sql ;
          update &RESLIB1..exposure_crm as a 
          set RW_EXP = (select b.RW_EXP from &RESLIB1..all_exp_ur_sec_std as b where a.EXP_ID=b.EXP_ID and b.RW_LOOK_THROUGH_UPDATE_FLG=1)
          where exists (select EXP_ID from &RESLIB1..all_exp_ur_sec_std as b where a.EXP_ID=b.EXP_ID and b.RW_LOOK_THROUGH_UPDATE_FLG=1)
          ;
        quit; 
    %end;

    %EXIT :
    
 /*- implement additional RW in the case of infringement of due diligence provisions.
    Assume that infringement can only happen when the bank is investing in a securitiation position -*/
    data &RESLIB1..all_exposures;
    set &RESLIB1..all_exposures;
        if SECURITIZATION_EXPOSURE_FLG =1 then RW_EXP = min(1250,RW_EXP);
        
        RW_EXP_BEFORE_INFRINGEMENT = RW_EXP;

        if SEC_INFRINGEMENT_FLG = 1 then do;
            RW_EXP = min(RW_EXP * 3.5, 1250);
        end;
        /* deduction treatment*/
        if SECURITIZATION_EXPOSURE_FLG =1 and RW_EXP = 1250 
          and (DEDUCT_FROM_OWN_FUNDS_FLG = 1 or IRB_PRODUCT_SUBCLASS = 'DEDUCTION' or STD_PRODUCT_SUBCLASS = 'DEDUCTION') then do;
            SEC_DEDUCTION_AMT = EAD;
            EAD = 0;
        end;
    run;
 
 /*- merge infringement information into &RESLIB1..exposure_crm for OPTIMAL method -*/
 data infringement_info(keep=EXP_ID RW_EXP_BEFORE_INFRINGEMENT);
    set &RESLIB1..all_exposures;
 run;

 proc sort data=infringement_info; by EXP_ID; run;
 
 data &RESLIB1..exposure_crm(drop=RW_LOOK_THROUGH_UPDATE_FLG);
         merge &RESLIB1..exposure_crm(in=exp_crm) infringement_info;
         by EXP_ID;
        if exp_crm;
 run;       
       
 /*- prepare data sets to include sec underlying exposures for RWA calculations -*/ 
 data &RESLIB1..all_exp_incl_sec_underly ;
    set &RESLIB1..all_exposures &RESLIB1..all_exp_sec_underlying ;
 run;
 
 proc sort data= &RESLIB1..all_exp_incl_sec_underly ;
    by exp_id ;
 run ;
 
 /*- update all_exposures table to exclude exposures with missing rw and securitization exposures not owned -*/
 data &RESLIB1..all_exposures;
     set &RESLIB1..all_exposures(where = (EXP_NOT_OWNED_FLG ne 1));
     %if &RETAIL_EXP_IND_LOAD_USFINAL = YES %then %do;  
                   if not missing(RW_EXP) ;
     %end;
     %else %do;
                   if not missing(OUTV_RW_SEG);
     %end;
     run;


%mend r00_post_regulatory_6_crd4;
