/***********************************************************************
 * Copyright (c) 2010-2012 by SAS Institute Inc., Cary, NC, USA.
 *
 * NAME: rdcube_main.sas
 *
 * PURPOSE: Builds an olap cube and information map.
 *
 * PARAMETERS:
 *    required parameters
 *    results_libref   = Libref containing RUNPROJECT results
 *    data_table       = Name of table from which to build results
 *    results_name     = Name for cube and information map (The underlying
 *                       data sets use this name as a prefix, so the name
 *                       may be truncated)
 *    analysis_type    = The analysis type specifies how to build the cube.
 *                       It affects dimension/hierarchies/aggregations for the cube
 *                       as well as the default measure dataset
 *                       If equals 'SUMMARY', then
 *                       If equals 'SIMULATION', then may include alphas,
 *                               horizons, output variables, and categories.
 *                       If equals 'SCENARIO', then may include output variables
 *                               and categories,
 *                       If equals 'SENSITIVITY', then may include output variables.
 *                       If equals 'SENSITIVITY2', then may include output variables.
 *
 *    olap_schema      = Olap Schema with which to register the cube and infomap
 *    out_folder       = Folder in which to place data sets, cube, and infomap
 *    out_metalib      = Metadata library in which to register cube data sets
 *
 *    optional parameters
 *    measure_table    = The analysis_type= sets appropriate default measures to include
 *                       in cube. But this option enables you to override that default.
 *    fileref?
 *    file_name        = Name of file containing code which generates the cube
 *                       and infomap. If this option is not specified, the fileref
 *                       is &results_libref..&analysis_type._&analysis_number.
 *    macro_name       = Name of macro in &file_name containing code which
 *                       generates the cube and infomap. If this option is
 *                       not specified, the name is &analysis_type._&analysis_number.
 *    description      = Description attributed to cube and infomap.
 *                       If description is not set, results_name is used.
 *    build_cube       = Specifies whether to build the cube as part of macro run.
 *                       If equals 0, do not run cube generation code.
 *                       Otherwise, run cube generation code.
 *                       Defaults to 1.
 *    rdcube_error            = If specified and an error occurs, this macro variable
 *                       will be set to 1. If no error occurs, it will be set
 *                       to 0.
 *
 ***********************************************************************/

%macro rdcube_main(
/* required arguments */
                       results_libref       = ,
                       data_table           = ,
                       drillthrough_table   = ,
                       results_name         = ,
                       olap_schema          = ,
                       out_folder           = ,
                       out_metalib          = ,
                       analysis_type        = ,

/* required if analysis_type ne "SUMMARY" */
                       analysis_number      = ,

/* optional arguments */
                       file_name            = build_cube.sas,
                       macro_name           = build_cube ,
                       build_cube           = 1,

                       description          = ,

                       analysis             = ,
                       measure_table        = ,

                       rdcube_error                = 
                       );

   %local out_path
          metaref_path
          description_used
          error_message
          argument_error
          wherelist
          sysvarlist
          write_detail_data
          ovname;

   /*****************************/
   /* miscellaneous preparation */
   /*****************************/
   %let RDCUBE_MSG = SASHELP.RDCUBE;
   %rdcube_check_arguments;
   %if &argument_error ne 0 %then %return;
   %rdcube_find_number_of_ccvars;
   %rdcube_make_filename( &results_libref );

   /***************************************/
   /* prepare filename to write cube code */
   /***************************************/
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   /***************/
   /* start macro */
   /***************/
   data _NULL_;
      file MyFile;
   put '%MACRO ' "&macro_name;";
   put '%local infomapExists olap_host;';

   /*********************/
   /* initialize &rdcube_error */
   /*********************/
   %if &rdcube_error ne %str() %then %do;
   put '%LET ' "&rdcube_error = 0;";
   %end;

   /***********************/
   /* table and cube work */
   /***********************/
   %rdcube_create_physical_libref;
   %rdcube_generate_table_code;
   %rdcube_generate_cube_code;
   %rdcube_generate_im_code;

   /*************/
   /* end macro */
   /*************/
   data _NULL_;
      file MyFile mod;
   put ' ';
   put "%MEND &macro_name;";
   run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   /*************************/
   /* run code if specified */
   /*************************/
   %if &build_cube ne 0 %then %do;
     %include "&out_path./&file_name";
     %&macro_name;
   %end;

%mend rdcube_main;

/********************/
/* auxiliary macros */
/********************/

%macro rdcube_check_arguments;
   %let argument_error = 0;
   %if &results_libref EQ %str() %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, _OUTPUT_= ));
      %let argument_error = 1;
   %end;
   %if &data_table EQ %str() %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, DATA_TABLE= ));
      %let argument_error = 1;
   %end;
   %if &drillthrough_table EQ %str() %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, DRILLTHROUGH_TABLE= ));
      %let argument_error = 1;
   %end;
   %if &results_name eq %str() %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, RESULTS_NAME= ));
      %let argument_error = 1;
   %end;
   %if "&olap_schema" eq "%str()" %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, OLAP_SCHEMA= ));
      %let argument_error = 1;
   %end;
   %if "&out_folder" eq "%str()" %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, OUT_FOLDER= ));
      %let argument_error = 1;
   %end;
   %if "&out_metalib" eq "%str()" %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, OUT_METALIB= ));
      %let argument_error = 1;
   %end;
   %if "&analysis_type" eq "%str()" %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_12, noquote, ANALYSIS_TYPE= ));
      %let argument_error = 1;
   %end;
   %if %symexist(rdcube_no_details) %then
      %let write_detail_data = 0;
   %else
      %let write_detail_data = 1;
   %if %upcase(&analysis_type) ne SCENARIO &&
       %upcase(&analysis_type) ne SENSITIVITY &&
       %upcase(&analysis_type) ne SENSITIVITY2 &&
       %upcase(&analysis_type) ne SIMULATION &&
       %upcase(&analysis_type) ne SUMMARY %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_10, noquote, &analysis_type ));
      %let argument_error = 1;
   %end;
   %if "&description" eq "%str()" %then
      %let description_used = &results_name;
   %else
      %let description_used = &description;
   %return;
   %if &argument_error ne 0 %then %do;
      data _NULL_;
      file MyFile;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_11, noquote ));
      run;
      %return;
      %end;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;
%mend rdcube_check_arguments;

%macro rdcube_find_number_of_ccvars;
   data _NULL_;
      set &results_libref..ccvars end=last;
      if last then do;
         call symput( "ccvarcount", _n_ );
         end;
      run;
   %let num_for_crossclass = %sysevalf(&ccvarcount*%sysfunc(log10(2))+1, ceil);
   %let num_left           = %eval(32 - &num_for_crossclass);
   %if %length(&results_name) + &num_for_crossclass > 32 %then %do;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_14, noquote,
                            &results_name, &num_left ));
      %return;
      %end;
%mend rdcube_find_number_of_ccvars;

%macro rdcube_make_filename( libref );
   %let oldgenopt=%sysfunc(getoption(generic));
   options nogeneric;
   /* how can I check for success here? */
   data _NULL_;
      z = pathname("&libref");
      call symput( "out_path", strip(z) );
      run;
   filename MyFile "&out_path/&file_name";
   options &oldgenopt;
%mend rdcube_make_filename;

%macro rdcube_create_physical_libref;
   data _NULL_;
      file MyFile mod;
      put " ";
      put "/* find physical libref for metadata library */";
      put '%let rskerr = 1;';
      put "DATA _NULL_;";
      put "   n = 1;";
      put "   LENGTH uri $500;";
      put "   LENGTH libref $32;";
      put "   uri = ' ';";
      put "   libref = ' ';";
      put '   nobj=METADATA_GETNOBJ("omsobj:SASLibrary?@Name=' "'&out_metalib'" '",n,uri);';
      put "   rc = METADATA_GETATTR(uri, 'libref', libref);";
      put "   CALL SYMPUT('rskerr',nobj-1);";
      put "   CALL SYMPUT('metaref',strip(libref));";
      put "   RUN;";
      %RDCUBE_CHECK_RSKERR1( 'rdcube_9', "&out_metalib" );
      run;
   %if &syserr ne 0 %then %return;

   data _NULL_;
      file MyFile mod;
      put " ";
      put "/* create local physical libref to metadata library */";
      put '%LET oldgenopt=%SYSFUNC(GETOPTION(GENERIC));';
      put "OPTIONS NOGENERIC;";
      put 'LIBNAME &metaref META LIBRARY="' "&out_metalib" '" METAOUT=DATA;';
      put 'OPTIONS &oldgenopt;';
      /* how to check for success here? */
      run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   data _NULL_;
      file MyFile mod;
      put ' ';
      put '/* find path to physical cube files */';
      put '%LET oldgenopt=%SYSFUNC(GETOPTION(GENERIC));';
      put 'OPTIONS NOGENERIC;';
      put 'data _NULL_;';
      put '   y=pathname("&metaref");';
      put '   if length(y) = 1 then do;';
      put '      call symput( "rskerr", 1 );';
      /*put '      put sasmsg( &RDCUBE_MSG, rdcube_13 , noquote, &metaref );';*/
      put '      end;';
      put '   else';
      put '      call symput( "metaref_path", strip(y) );';
      put '   run;';
      put 'OPTIONS &oldgenopt;';
      %RDCUBE_CHECK_RSKERR1( 'rdcube_3', '&metaref' );
      run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;

   data _NULL_;
      file MyFile mod;
      put " ";
      put "/* create local physical libref to metadata library */";
      put '%LET oldgenopt=%SYSFUNC(GETOPTION(GENERIC));';
      put "OPTIONS NOGENERIC;";
      put 'LIBNAME &metaref "&metaref_path";';
      put 'OPTIONS &oldgenopt;';
      /* how to check for success here? */
      run;
   %rdcube_check_syserr;
   %if &syserr ne 0 %then %return;
%mend rdcube_create_physical_libref;

/*************************/
/* error handling macros */
/*************************/

/* use CAPS in macro name to emphasize it writes to generated file. */
%macro RDCUBE_SET_RSKERR;
   put '%if &syserr ne 0 AND &syserr ne 4 %then';
   put '   %let rskerr = 1;';
   put '%else';
   put '   %let rskerr = 0;';
%mend RDCUBE_SET_RSKERR;

/* use CAPS in macro name to emphasize it writes to generated file. */
%macro RDCUBE_CHECK_RSKERR( error_message );
   put '%if &rskerr ne 0 %then %do;';
   put '  %put %sysfunc(sasmsg(' " &RDCUBE_MSG, " &error_message ', noquote ));';
   %if &rdcube_error ne %str() %then %do;
      put '   %let ' "&rdcube_error = 1;";
      %end;

   put '   %return;';
   put '   %end;';
%mend RDCUBE_CHECK_RSKERR;

%macro RDCUBE_CHECK_RSKERR1( error_message, my_string_argument );
   put '%if &rskerr ne 0 %then %do;';
   put '  %put %sysfunc(sasmsg(' " &RDCUBE_MSG, " &error_message ', noquote, "' &my_string_argument '" ));';
   %if &rdcube_error ne %str() %then %do;
      put '   %let ' "&rdcube_error = 1;";
      %end;

   put '   %return;';
   put '   %end;';
%mend RDCUBE_CHECK_RSKERR1;

%macro rdcube_check_syserr( error_message );
   %if &syserr ne 0 %then %do;
      %if &error_message ne %str() %then %do;
         %put "&error_message";
         %end;
      %put %sysfunc(sasmsg( &RDCUBE_MSG, rdcube_11, noquote ));
      %return;
      %end;
%mend rdcube_check_syserr;
